/*
* $Log: army.c,v $
* Revision 1.6  2004/02/18 19:57:40  richq
* Fey advantage in forests was incorrect
*
* Revision 1.5  2004/01/30 22:30:29  Default
* Dynamic memory stuff removed - replaced by static ewram assignment.
*
* Revision 1.4  2004/01/27 20:19:12  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.3  2004/01/25 21:04:44  richq
* Recruitment stuff + bios divide
*
* Revision 1.2  2004/01/22 20:09:07  richq
* Now fills battle tables for a location. Func to describe armies in words added.
*
* Revision 1.1  2004/01/21 19:15:36  Default
* Army contains code for the garrisons (armies at Keeps and Citadels) and
* for Doomdarks wandering regiments.
*
*/

#include "qmath.h"
#include "stdlib.h"
#include "army.h"
#include "places.h"
#include "icefear.h"
#include "lords.h"
#include "tokens.h"
#include "tokendata.h"

#include "memory_defs.h"

#define garrisonROMData _binary_garrisondata_bin_start
extern u8 garrisonROMData [];

#define regimentROMData _binary_regimentdata_bin_start
extern u8 regimentROMData [];

garrison_t * const g_garrisons = (garrison_t *)ADDR_G_GARRISONS;
regiment_t * const g_regiments = (regiment_t *)ADDR_G_REGIMENTS;

int g_doomdarkWarriors;
int g_doomdarkRiders;

int g_freeindex;
army_t * const g_freeTable = (army_t *)ADDR_G_FREETABLE;
int g_doomindex;
army_t * const g_doomTable = (army_t *)ADDR_G_DOOMTABLE;

int g_lordsHere;
u8 g_charTable[32];

// modifier for when there is a garrison at a location
int g_freeguard;
int g_doomguard;
int g_riderEnergyMod;

// total free (free armies + lords) at a location
int g_totalFree;
// total armies (free + doom) at a location
int g_totalArmies;

int g_tmpGarrisonID;

void init_garrisons(void) {
  
  int i;
  int index = 0;
  for (i = 0; i < 102; i++) {
    g_garrisons[i].type = (garrisonROMData[index]&4)/4; // 1 for riders, 0 for warriors
    g_garrisons[i].race = garrisonROMData[index]&3;
    index++;
    g_garrisons[i].size = garrisonROMData[index++];
  }
  
}

void increase_ddArmy(int size, int type) {
  if (type == 0)
    g_doomdarkWarriors += size;
  else
    g_doomdarkRiders += size;
}

// add the doomguard entry to the battle table
// code based on e7d4
void addDoomdarkBattleTable(army_t * doomguard, int type) {
  
  signed long chanceModifier;
  signed long unused;
  if (type) {
    // riders
    chanceModifier = icefear / 4;
  } else {
    // warriors
    //chanceModifier = icefear/5;
    FastIntDivide(icefear, 5, &chanceModifier, &unused);
    
  }
  doomguard->chance = chanceModifier + g_doomguard;
  // store in table...
  g_doomTable[g_doomindex++] = *doomguard;
}

// based on e7a4
void addFreeBattleTable(army_t * freearmy) {
  
  freearmy->chance += g_freeguard;
  
  // check the super modifier that is applied for Fey in woods
  if (g_locdat.terrain == FOREST && g_selectedLord->graphic >= 0xf) {
    freearmy->chance += 64;
  }
  freearmy->chance /= 2;
  freearmy->chance += 0x18;
  freearmy->killed = 0;
  
  g_freeTable[g_freeindex++] = *freearmy;
}


// based on code at e6d1
// we are at a special location and there is a garrison here.
void addGarrisonToBattleTable(int locationID, garrison_t * garrison, locdat_t * locdat) {
  // modifies chance based on terrain
  // default is 16 (for keeps, I guess)
  int terrainModifier = 16;
  if (locdat->terrain == CITADEL) {
    terrainModifier = 32;
  }
  
  if (garrison->race == RACE_DOOMDARK ) {
    // e6e3:
    increase_ddArmy(garrison->size,garrison->type);
    // call e7d4
    army_t doomguard;
    doomguard.id = locationID|0x80;
    doomguard.size = garrison->size;
    doomguard.killed = 0;
    doomguard.chance = 0;
    g_doomguard = terrainModifier;
    addDoomdarkBattleTable(&doomguard, garrison->type);
    
  } else {
    // e70a...
    army_t freearmy;
    freearmy.id = locationID|0x80;
    freearmy.size = garrison->size;
    g_freeguard = terrainModifier;
    freearmy.killed = 0;
    freearmy.chance = 0x40;
    if (garrison->type) {  // riders are better
      freearmy.chance = 0x60;
    }
    // store by value. I hope.
    g_freeTable[g_freeindex++] = freearmy;
  }
  
  
  
}

// based on e73f
// adds a wandering doomguard regiment to the doomdark battle table
void addRegimentToBattle(int regimentid) {
  army_t doomguard;
  doomguard.id = regimentid;
  doomguard.size = g_regiments[regimentid].size;
  doomguard.killed = 0;
  doomguard.chance = 0;
  increase_ddArmy(g_regiments[regimentid].size,g_regiments[regimentid].type);
  addDoomdarkBattleTable(&doomguard, g_regiments[regimentid].type);
}

// based on e75c
// adds lord (currently in the temp lord variable) to the battle
void addLordToBattle(void) {
  
  // increase count of lords at this square and store their id
  g_charTable[g_lordsHere++] = g_selectedLordID;
  
  army_t lordCommands;
  if (g_selectedLord->riders > 0) {
    lordCommands.size = g_selectedLord->riders;
    lordCommands.id = g_selectedLordID|0x40;
    lordCommands.chance = g_riderEnergyMod + g_selectedLord->riders_energy;
    // call e7a4
    addFreeBattleTable(&lordCommands);
  }
  if (g_selectedLord->warriors > 0) {
    lordCommands.size = g_selectedLord->warriors;
    lordCommands.id = g_selectedLordID;
    lordCommands.chance = g_selectedLord->warriors_energy;
    // call e7a4
    addFreeBattleTable(&lordCommands);
  }
  
  
}



void init_army(void) {
  init_garrisons();
  
  int i;
  int index = 0;
  u8 theByte;
  // copy rom data to fill up regiments
  for (i = 0; i < 128; i++) {
    theByte = regimentROMData[index++];
    g_regiments[i].orders = theByte & 0xc0;
    g_regiments[i].x = theByte & 0x3f;
    theByte = regimentROMData[index++];
    g_regiments[i].type = theByte &0x80;
    g_regiments[i].y = theByte & 0x3f;
    g_regiments[i].size    = regimentROMData[index++];
    g_regiments[i].id      = regimentROMData[index++];
    
  }
    
}

//ef3f
// "one thousand riders of the free guard the keep"
void addArmyTokens(void) {
  
  if (g_doomindex) {
    addToTokenList(doomdarkHasArmy);
    numberToWords(g_doomdarkWarriors*5);
    addSingleToken(0xc5); //warrior
    addLiteralToken('s');
    addSingleToken(0x87); // and
    numberToWords(g_doomdarkRiders*5);
    addSingleToken(0xc4); //rider
    addLiteralToken('s'); 
    addLiteralToken('.');
    if (g_locdat.terrain == CITADEL || g_locdat.terrain == KEEP) {
      addUppercaseToken();
    }
    else return;
    
  }
  // add the guards of the free (will only get here if a keep's in front of us)
  // g_tmpGarrisonID stores the garrison id
  if (g_garrisons[g_tmpGarrisonID].type) {
    numberToWords(g_garrisons[g_tmpGarrisonID].size*5);
    addSingleToken(0xc4); //rider
    addLiteralToken('s'); 
  } else {
    numberToWords(g_garrisons[g_tmpGarrisonID].size*5);
    addSingleToken(0xc5); //warrior
    addLiteralToken('s'); 
  }
  addToTokenList(ofthe);
  
  // which race?
  if (g_garrisons[g_tmpGarrisonID].race == 0) {
    addToTokenList(doomGrdTokens);
  } else if (g_garrisons[g_tmpGarrisonID].race == 1) {
    addSingleToken(0x9f);
  } else if (g_garrisons[g_tmpGarrisonID].race == 2) {
    addSingleToken(0x7a);
  } else  {
    addSingleToken(0x3a);
  }
  
  addToTokenList(guardThe);
  addSingleToken(0x41+g_locdat.terrain); // keep/citadel
  
  addLiteralToken('.');
  
}

// positive amount == men go with lord (recruit)
// negative amount == men posted to garrison (guard keep)
void swapWithGarrison(int amount) {
  // remove 100 men (100/5 = 20) from the keep
  g_garrisons[g_tmpGarrisonID].size -= amount;
  if (g_garrisons[g_tmpGarrisonID].type) {
    g_selectedLord->riders += amount;
  } else {
    g_selectedLord->warriors += amount;
  }
  
}
