/*
* $Log: army.h,v $
* Revision 1.5  2004/01/31 15:20:30  Default
* func prototype added to prevent no-declaration warning
*
* Revision 1.4  2004/01/30 22:30:29  Default
* Dynamic memory stuff removed - replaced by static ewram assignment.
*
* Revision 1.3  2004/01/25 21:04:44  richq
* Recruitment stuff + bios divide
*
* Revision 1.2  2004/01/22 20:09:07  richq
* Now fills battle tables for a location. Func to describe armies in words added.
*
* Revision 1.1  2004/01/21 19:15:36  Default
* Army contains code for the garrisons (armies at Keeps and Citadels) and
* for Doomdarks wandering regiments.
*
*/


#ifndef _ARMY_H
#define _ARMY_H


#include "gba.h"
#include "location.h"


#define MAX_DOOMDARK_TABLE 32
#define MAX_FREE_TABLE 32

#define RACE_DOOMDARK 0
#define RACE_FREE 1

#include "gba.h"

/*
doomdarks 128 wandering regiments
these get initialised with certain orders, locations etc
*/

typedef struct {
  u8 orders;
  u8 x;
  u8 type;
  u8 y;
  u8 size;
  u8 id;   /* "target"? */ 
}
regiment_t;

/* 
102 garrisons. 
These are the soldier in the citadels and keeps
they get killed/recruited, so size may well be 0 if all have gone
they get initialised with default values
*/
typedef struct {
  u8 type;
  u8 race;
  u8 size;
}
garrison_t;


/*
The armies of the free and doomdark at a location

*/
typedef struct {
  // size is the size of the force at a location
  u8 size; 
  // chance for the free depends on size, type etc
  // chance for doomdark depends on type and icefear
  u8 chance;
  // killed is number of enemies killed by the army in battle
  u8 killed;
  // id is the index of the garrison |0x80 
  // or the regiment id (doomdark only)
  u8 id;
}
army_t;

void init_army(void);

// list of 128 doomguard regiments
extern regiment_t * const g_regiments;
// list of 102 garrisons, each represents the guard of a special location
extern garrison_t * const g_garrisons;

// the id of the garrison in the square we are in 
// is only valid if the square is a keep or citadel, mind.
extern int g_tmpGarrisonID;

// table of free armies at a location
extern int g_freeindex;
extern army_t * const g_freeTable;
// table of doomguard armies at a location
extern int g_doomindex;
extern army_t * const g_doomTable;

// table of lords at a location

extern int g_lordsHere;   // number of lords here
extern u8 g_charTable[32];

// count of total number off DDs warriors and riders at a location
extern int g_doomdarkWarriors;
extern int g_doomdarkRiders;
// a modifier for when there is a garrison at a location
extern int g_freeguard;
extern int g_doomguard;
// a modifier for rider energy - more tired in the mountains
extern int g_riderEnergyMod;

// total free (free armies + lords) at a location
extern int g_totalFree;
// total armies (free + doom) at a location
extern int g_totalArmies;


// add a garrison to its appropriate battle table (free or doom)
void addGarrisonToBattleTable(int locationID, garrison_t * garrison, locdat_t * locdat);
// add a wandering doomguard regiment to the doom battle table
void addRegimentToBattle(int regimentid);
void addArmyTokens(void);

void addLordToBattle(void);

// positive amount == men go with lord (recruit)
// negative amount == men posted to garrison (guard keep)
void swapWithGarrison(int amount);

#endif

