/*
* $Log: doomdark.c,v $
* Revision 1.6  2004/02/18 19:58:19  richq
* const'd lakeMirrow
*
* Revision 1.5  2004/01/31 18:35:31  Default
* Fix menu highlighter bug after night is called from another menu
*
* Revision 1.4  2004/01/30 22:33:29  Default
* gameover flag added
*
* Revision 1.3  2004/01/27 20:19:13  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.2  2004/01/25 21:11:02  richq
* All sorts of fighting added - wolves, Doomdark, nighttime movements.
*
* Revision 1.1  2004/01/22 22:23:37  richq
* Night time and things to do with Doomdarks movements are in here.
*
*/

#include "qmath.h"
#include "doomdark.h"
#include "gfx.h"
#include "shield.h"
#include "objects.h"
#include "tokens.h"
#include "tokendata.h"
#include "savearea.h"
#include "army.h"
#include "places.h"
#include "lords.h"
#include "icefear.h"
#include "look.h"
#include "inter.h"
#include "menu.h"

int g_doomdarkCitadels;

int l_tmpTotalArmies;

void flagLordPositions(void) {
  int i;
  
  for (i = 0; i < 32; i++) {
    setSelectedLord(i);
    if (g_selectedLord->alive && !g_selectedLord->hidden) {
      // set special flag
      g_locdat.x = g_selectedLord->location.x;
      g_locdat.y = g_selectedLord->location.y;
      fillLocationData(&g_locdat);
      g_locdat.special = 0x80;
      saveLocationData();
    }
  }
}

void flagFreeKeeps() {
  int i;
  for (i = 0; i < 0x66; i++) {
    if (g_garrisons[i].race != 0) {
      g_locdat.x = specialPlacesTable[i].x;
      g_locdat.y = specialPlacesTable[i].y;
      fillLocationData(&g_locdat);
      g_locdat.special = 0x80;
      saveLocationData();
    }
  }
  
}

void makeArmyStay(int regid,int * move) {
  // set the location data to the regiment position
  g_locdat.x = g_regiments[regid].x;
  g_locdat.y = g_regiments[regid].y;
  
  // fill in the gaps
  fillLocationData(&g_locdat);
  // update the tile data here with army
  setTileToArmy(1);
  *move = 8;
}


/*
  loc: the location we want to go to
  regid : the regiment that is currntly moving
  move : the current movement allowance
  
  returns the direction to move in, or 8 if nothing really possible
  also fills g_inFront by calling lookingAt
  */
int getBestDirection(location_t * const target, int regid, int * move) {
  game_frames = 0;
  location_t start;
  start.x = g_regiments[regid].x;
  start.y = g_regiments[regid].y;
  
  int modx = 0;
  int mody = 0;
  
  if (target->x < start.x) {
    modx--;
  } else if (target->x > start.x) {
    modx++;
  }
  
  if (target->y < start.y) {
    mody--;
  } else if (target->y > start.y) {
    mody++;
  }
  
  // now find which direction we are going to move in
  int dir;
  for (dir = 0; dir < 8; dir++) {
    if (positionModifier[dir][0] == modx && positionModifier[dir][1] == mody)
      break;
  }
  // no way dir can be > 8
  int dirTable[4];
  dirTable[0] = dir;
  dirTable[1] = dir;
  dirTable[2] = (dir-1)&7; // will be 7 if < 0
  dirTable[3] = (dir+1)&7;
  
  int index;
  int i;
  for (i = 0; i < 8; i++) {
    index = randomish()&3;
    start.direction = dirTable[index];
    lookingAt(&start);
    g_locdat.x = g_inFront.x;
    g_locdat.y = g_inFront.y;
    fillLocationData(&g_locdat);
    
    if (g_locdat.terrain != MOUNTAIN &&
        g_locdat.terrain != FOREST &&
        g_locdat.terrain != WASTES) {
      return start.direction;
    }
  }
  
  return (g_locdat.terrain == WASTES)?8:start.direction;
  
}



//eb4e
void checkInfrontOfArmy(int looking,int regid,int * move) {
  if (looking >= 8) {
    makeArmyStay(regid,move);
    return;
  }
  
  workOutLocationDetails(&g_inFront);
  if (g_doomindex >= 0x1f) {
    // the infront location is already full
    makeArmyStay(regid,move);
    return;
  }
  // else move army...
  
  int x,y;
  x = g_locdat.x;
  y = g_locdat.y;
  
  g_locdat.x = g_regiments[regid].x;
  g_locdat.y = g_regiments[regid].y;
  fillLocationData(&g_locdat);
  l_tmpTotalArmies--;       // decrease the number on this square
  setTileToArmy(l_tmpTotalArmies);
  l_tmpTotalArmies = g_totalArmies + 1;
  
  g_regiments[regid].x = x;
  g_regiments[regid].y = y;
  g_locdat.x = g_regiments[regid].x;
  g_locdat.y = g_regiments[regid].y;
  
  fillLocationData(&g_locdat);
  
  // check movement cost
  int cost = 2;
  
  if (g_locdat.terrain == MOUNTAIN || g_locdat.terrain == FOREST) {
    cost = 8;
  }
  if (g_regiments[regid].type) 
    cost /= 2;
  
  *move += cost;
  
  setTileToArmy(1);

}

void attemptFollow(location_t * const target, int regid, int *move) {
  if (g_regiments[regid].x == target->x
   && g_regiments[regid].y == target->y) {
    makeArmyStay(regid, move);
  } else {
    // follow
    // e297 move towards something
    int direction = getBestDirection(target, regid, move);
    checkInfrontOfArmy(direction,regid,move);
  }
}



// move regiment regid somewhere
//eb07
void doArmyMove(int regid, int * move, location_t * armyLoc) {
  
  if (g_regiments[regid].size) {
    // we are an army of considerable size.
    if (g_locdat.special != 0) {
      // and this spot is interesting, stay here
      makeArmyStay(regid,move);
      return;
    } else {
      // no one here... look around
      
      int dir;
      locdat_t locdat;
      for (dir = 0; dir < 8; dir++) {
        armyLoc->direction = dir;
        lookingAt(armyLoc);
        locdat.x = g_inFront.x;
        locdat.y = g_inFront.y;
        fillLocationData(&locdat);
        
        if (locdat.special) {  
          // something is in front of us
          // eb4e
          checkInfrontOfArmy(dir,regid,move);
          return;
        }
      }
      
    }
    // We've checked around us, now check our orders
    if (g_regiments[regid].orders == 0) {
      
      // move towards special location if an army's there
      g_locdat.x = specialPlacesTable[g_regiments[regid].id].x;
      g_locdat.y = specialPlacesTable[g_regiments[regid].id].y;
      fillLocationData(&g_locdat);
      if (!g_locdat.special) { 
        //stay here - only move if the free attack the target
        makeArmyStay(regid, move);
        return;
      }
      // attempt follow
      location_t loc;
      loc.x = specialPlacesTable[g_regiments[regid].id].x;
      loc.y = specialPlacesTable[g_regiments[regid].id].y;
      
      attemptFollow(&loc, regid, move);
      return;
    }
    
    if (g_regiments[regid].orders == 0x40) {
      // move to new random loc
      location_t start;
      do {
        
        start.x = g_regiments[regid].x;
        start.y = g_regiments[regid].y;
        start.direction = randomish()& 7;
        lookingAt(&start);
        
        g_locdat.x = g_inFront.x;
        g_locdat.y = g_inFront.y;
        fillLocationData(&g_locdat);
      } while (g_locdat.terrain == WASTES);
      
      checkInfrontOfArmy(start.direction,regid,move);
      return;
    }
    
    if (g_regiments[regid].orders == 0x80) {
      // follow lord..
      setSelectedLord(g_regiments[regid].id);
      if (!g_selectedLord->alive) {
        g_regiments[regid].id = LUXOR;
        setSelectedLord(g_regiments[regid].id);
        if (!g_selectedLord->alive) {
          g_regiments[regid].id = MORKIN;
          setSelectedLord(g_regiments[regid].id);
        }
      }
      
      // attempt follow
      attemptFollow(&g_selectedLord->location, regid, move);
      return;
      
    }
    
    if (g_regiments[regid].orders == 0xc0) {
      location_t loc;
      if (g_regiments[regid].x != specialPlacesTable[g_regiments[regid].id].x ||
          g_regiments[regid].y != specialPlacesTable[g_regiments[regid].id].y)
      {
        // attempt follow
        loc.x = specialPlacesTable[g_regiments[regid].id].x;
        loc.y = specialPlacesTable[g_regiments[regid].id].y;
        int direction = getBestDirection(&loc, regid, move);
        checkInfrontOfArmy(direction,regid,move);
        return; 
      } else {
        //we've arrived!!
        
        int newTarget = specialPlacesTable[g_regiments[regid].id].left;
        if (randomish() >= 0x80) {
          newTarget = specialPlacesTable[g_regiments[regid].id].right;
        }
        g_regiments[regid].id = newTarget;
        loc.x = specialPlacesTable[g_regiments[regid].id].x;
        loc.y = specialPlacesTable[g_regiments[regid].id].y;
        int direction = getBestDirection(&loc, regid, move);
        checkInfrontOfArmy(direction,regid,move);
        return; 
        
      }
    }
  } else {
    *move = 8;
  }
  
}



//ec5d
void processDoomguard()  {
  int i,move;
  location_t tmploc;
  
  // loop on regiments of the doomguard
  for (i = 0; i < 128; i++) {
    tmploc.x = g_regiments[i].x;
    tmploc.y = g_regiments[i].y;
    workOutLocationDetails(&tmploc);
    l_tmpTotalArmies = g_totalArmies;
    // 6 moves per army
    game_frames = 0;
    for (move = 0; move < 6; move++) {
      doArmyMove(i, &move, &tmploc);
      tmploc.x = g_regiments[i].x;
      tmploc.y = g_regiments[i].y;
    }
    
    
  }
  
  
}

int l_armiesOfA;
signed long l_armiesOfB;
army_t * l_TableOfA;
army_t * l_TableOfB;

void shuffleArmyTable(int removed) {
  // shuffle the army table when we have just killed an army
  // place the removed army at the end
  // armies always removed from B
  if (l_armiesOfB == removed) 
    return; // removed the last entry, no need to do anything
  else {
    // need to shuffle table down
    army_t tmp = l_TableOfB[removed];
    int i;
    for (i = removed; i < l_armiesOfB; i++) {
      l_TableOfB[i] = l_TableOfB[i+1];
    }
    l_TableOfB[l_armiesOfB] = tmp;
  }
  
}


void rollBattle(army_t * attacker) {
  int i;
  signed long hitid, result;
  for (i = 0; i < attacker->size; i++) {
    if (randomish() < attacker->chance) {
      // successfull hit
      // calculate which army we hit
      FastIntDivide(randomish(), l_armiesOfB, &result, &hitid);
      //hitid = randomish() % l_armiesOfB;
      
      if (l_TableOfB[hitid].chance < randomish()) {
        // hit!
        if (attacker->killed < 255)
          attacker->killed++;
        l_TableOfB[hitid].size--;
        if (l_TableOfB[hitid].size == 0) {
          l_armiesOfB--;
          if (l_armiesOfB == 0) {
            return; // we've won
          } else {
            shuffleArmyTable(hitid); 
          }
        }
        
      }
            
    }
    
    
  }
  
  
}



void calculateLordKills() {
  if (!g_lordsHere)
    return;
  army_t lordAsArmy;
  // loop on lords here
  int i;
  for (i = 0; i < g_lordsHere; i++) {
    if (l_armiesOfB == 0)
      return;
    setSelectedLord( g_charTable[i] );
    
    lordAsArmy.size = g_selectedLord->strength;
    lordAsArmy.chance = g_selectedLord->energy + 0x80;
    lordAsArmy.killed = 0;
    rollBattle(&lordAsArmy);
    g_selectedLord->killed = lordAsArmy.killed;
  }
  
  
}

void swapSides() {
  int tmpVal = l_armiesOfA;
  l_armiesOfA = l_armiesOfB;
  l_armiesOfB = tmpVal;
  
  army_t * tmpArmy = l_TableOfA;
  l_TableOfA = l_TableOfB;
  l_TableOfB = tmpArmy;
}

void battleOneSide() {
  if (l_armiesOfA == 0 || l_armiesOfB == 0) 
    return;
  
  int i;
  for (i = 0; i < l_armiesOfA && l_armiesOfB; i++) {
    if (l_TableOfA[i].size) {
      rollBattle(&l_TableOfA[i]);
    }
  }
  
}


void battleBothSides() {
  
  battleOneSide();
  swapSides();
  battleOneSide();
  swapSides();
  
}


void calculateCitadelLosses(int whoWon, army_t * guards) {
  int garrisonid = guards->id & 0x7f;
  int newSize = guards->size;
  if (whoWon == 0xc8) {     // enemy won
    if (g_garrisons[garrisonid].race != 0) {
      g_garrisons[garrisonid].race = 0;
      newSize = 50;
    }
  } else if (whoWon != 0) { // free won
    if (g_garrisons[garrisonid].race == 0) {
      g_garrisons[garrisonid].race = 1;
      newSize = 40;
    }
  }
  if (!newSize) {   // maybe the battle continues, but the garrison is dead
                    // add a handful of reinforcements
    newSize = 4;
  }
  
  g_garrisons[garrisonid].size = newSize;
  
}


void whoWon(int battleArea) {
  //e8c1
  int i;
  int newEnergy;
  int wonToken = 0; //The
  if (!l_armiesOfB) {     // B is doomdark, none left
    //token 9f
    wonToken = 0x9f; //Free
  } else if (!l_armiesOfA) {  // A is free, none left
    // token c8
    wonToken = 0xc8; //Enemy
  }
  if (g_freeindex) {
    // free army intially non zero
    
    for (i = 0; i < g_freeindex; i++) {
      if (g_freeTable[i].id & 0x80) {
        // army is a garrison
        // call e99e
        calculateCitadelLosses(wonToken, &g_freeTable[i]);
      } else {
        // army is a lord's
        setSelectedLord(g_freeTable[i].id&0x1f);
        if (g_freeTable[i].id&0x40) {
          // riders
          g_selectedLord->riders_lost = g_selectedLord->riders - g_freeTable[i].size;
          g_selectedLord->riders = g_freeTable[i].size;
          g_selectedLord->riders_killed = g_freeTable[i].killed;
          newEnergy = g_selectedLord->riders_energy - 0x18;
          g_selectedLord->riders_energy = (newEnergy<0)?0 : newEnergy;
          
        } else {
          // warriors
          g_selectedLord->warriors_lost = g_selectedLord->warriors - g_freeTable[i].size;
          g_selectedLord->warriors = g_freeTable[i].size;
          g_selectedLord->warriors_killed = g_freeTable[i].killed;
          newEnergy = g_selectedLord->warriors_energy - 0x18;
          g_selectedLord->warriors_energy = (newEnergy<0)?0 : newEnergy;
          
        }
      }
    }
  }
  
  if (g_doomindex) {
    
    for (i = 0; i < g_doomindex; i++) {
      if (g_doomTable[i].id & 0x80) {
        calculateCitadelLosses(wonToken, &g_doomTable[i]);
      } else {
        // wandering regiment
        g_regiments[g_doomTable[i].id].size = g_doomTable[i].size;
      }
    }
  }
  
  // now sort out lords...
  if (g_lordsHere) {
    
    for (i = 0; i < g_lordsHere; i++) {
      setSelectedLord( g_charTable[i] );
      g_selectedLord->battle_status = wonToken;
      g_selectedLord->battle_area = battleArea;
      newEnergy = g_selectedLord->energy - 20;
      g_selectedLord->energy = (newEnergy<0)?0 : newEnergy;
      
    }
  }
  
  if (wonToken == 0xc8) {
    // enemy won.. ea0c
    for (i = 0; i < g_lordsHere; i++) {
      setSelectedLord( g_charTable[i] );
      fightEnemy();
      if (g_selectedLord->alive) {
        // we lost, but survived... run away!
        location_t escapeTo;
        escapeTo = g_selectedLord->location;
        do {
          escapeTo.direction = randomish()&7;
          // look at what we want to escape to
          lookingAt(&escapeTo);
          g_locdat.x = g_inFront.x;
          g_locdat.y = g_inFront.y;
          fillLocationData(&g_locdat);
        }
        while (g_locdat.terrain == WASTES);
        g_selectedLord->location = g_inFront;
        
        
      }
    }
    
  }
}

int totalBattles;
void battlesOccurred(int area) {
  //edf1
  totalBattles++;
  if (totalBattles == 1) {
    addToTokenList(theBloodySword);
    
  } else {
    addSingleToken(0x87); // and
  }
  addSingleToken(0x40); // of
  
  // create area name
  addAreaToken(area);
  
  print_tokens_pal(3);
  
}

/*
void debug_battle() {
  int i;
  
  for (i = 0; i < g_freeindex; i++) {
    gbaprintf("Free %d [%d]",i,g_freeTable[i].size);
  }
  for (i = 0; i < g_doomindex; i++) {
    gbaprintf("Doom %d [%d]",i,g_doomTable[i].size);
  }
  
}
*/

void swordOfBattle() {
  
//  gbaprintf("Battle @ %d %d",g_locdat.x,g_locdat.y);
//  gbaprintf("Free %d and %d lords",g_freeindex,g_lordsHere);
//  gbaprintf("Doom %d ",g_doomindex);
//  debug_battle();
  
  int area = g_locdat.area;
  l_armiesOfA = g_freeindex;
  l_armiesOfB = g_doomindex;
  
  l_TableOfA = g_freeTable;
  l_TableOfB = g_doomTable;
  
  

  calculateLordKills();
  
  battleBothSides();
  battleBothSides();
  
//  gbaprintf("Fight over ");
//  debug_battle();
  
  whoWon(area);
  // do stuff with area "bloody sword of battle..."
  battlesOccurred(area);
  
}

  

// check if we need to battle
void checkBattles() {
  int i;
  int tmpSpecial;
  location_t tmpLoc;
  totalBattles = 0;
  for (i = 0; i < 32; i++) {
    setSelectedLord(i);
    // unset special flag
    g_locdat.x = g_selectedLord->location.x;
    g_locdat.y = g_selectedLord->location.y;
    fillLocationData(&g_locdat);
    tmpSpecial = g_locdat.special;
    g_locdat.special = 0;
    saveLocationData();
    
    if (tmpSpecial) {
      tmpLoc.x = g_selectedLord->location.x;
      tmpLoc.y = g_selectedLord->location.y;
      workOutLocationDetails(&tmpLoc);
      if (g_doomindex) {
        // fight!
        // e87d
        swordOfBattle();
      }
      
    }
    
  }
  
  for (i = 0; i < 0x66; i++) {
    // unset special flag
    g_locdat.x = specialPlacesTable[i].x;
    g_locdat.y = specialPlacesTable[i].y;
    fillLocationData(&g_locdat);
    tmpSpecial = g_locdat.special;
    g_locdat.special = 0;
    saveLocationData();
    
    if (tmpSpecial) {
      tmpLoc.x = specialPlacesTable[i].x;
      tmpLoc.y = specialPlacesTable[i].y;
      workOutLocationDetails(&tmpLoc);
      if (g_doomindex) {
        // fight!
        //e87d
        swordOfBattle();
      }
    }
  }
  
}



void doDoomguardMoves(void) {
  //66ad
  
  flagLordPositions();
  flagFreeKeeps();
  
  processDoomguard();
  //66cc
  checkBattles();
  
}


void calculateDDCitadels(void) {
  
  locdat_t locdat;
  int i;
  g_doomdarkCitadels = 0;
  for (i = 0; i < 102; i++) {
    if (g_garrisons[i].race == 0) {
      // it is a dd garrison...
      locdat.x = specialPlacesTable[i].x;
      locdat.y = specialPlacesTable[i].y;
      fillLocationData(&locdat);
      if (locdat.terrain == CITADEL)
        g_doomdarkCitadels += 5;
      else
        g_doomdarkCitadels += 2;
    }
  }
}

void setupNightScreen(void) {
  // clear top screen
  clear_text();
  // clear bottom
  clear_objects();
  clearHighlighter();
  // clear shield...
  remove_shield();
  clear_view();
  CopyOAM();
  set_night_palette();
  clearBuffer();
}

const location_t lakeMirrow = {
  0x9, 0x12, 0
};


// returns 1 if it's game over (win or lose)
int checkVictoryDefeat() {
  // check if L and M are dead or not...
  
  int luxmor = 0;
  int icecrown = 0;
  setSelectedLord(LUXOR);
  if (!g_selectedLord->alive) {
    luxmor++;
    if (g_selectedLord->object == OBJECT_MOON_RING) {
      // drop the moon ring
      g_selectedLord->object = 0;
      
      fillLocationData(&g_locdat);
      g_locdat.object = OBJECT_MOON_RING;
      saveLocationData(); 
      // lose control of lords...
      toggleControl(1);
      
    }
  }
  
  // now check morks
  setSelectedLord(MORKIN);
  if (!g_selectedLord->alive) {
    luxmor^=2;
    // check game over...
  }
  else {
    // MOrkin is alive
    if (g_selectedLord->object == OBJECT_MOON_RING) {
      // regain control..
      toggleControl(3);
      // check game over
    } else if (g_selectedLord->object == OBJECT_ICE_CROWN) {
      // are we able to destroy the ice crown?
      if (g_selectedLord->location.x == lakeMirrow.x &&
          g_selectedLord->location.y == lakeMirrow.y) {
            icecrown = 1;
            
      }
      int x = g_selectedLord->location.x;
      int y = g_selectedLord->location.y;
      if (!icecrown) {
        // not at lake mirrow...
        int i;
        for (i = 0x1d ; i < 32; i++) {
          setSelectedLord(i);
          if (g_selectedLord->alive &&
              g_selectedLord->location.x == x &&
              g_selectedLord->location.y == y) {
            icecrown = 1;
            break;
          }
        }
      }
    } // end if moon ring/ice crown
  }
  // check game over...
  //8f42..
  // first check Doomdarks victory...
  if (luxmor == 3) {
    // dead
    addToTokenList(luxorsdead);
    addToTokenList(andMorkinIsDead);
    print_tokens(0,0,0,0x1e,3);
    return 1;
  }
  if (luxmor == 2 && (g_garrisons[0x60].race == 0)) {
    // morkin is dead, Xajorkith is Doomdarks...
    addToTokenList(xajHasFallen);
    addToTokenList(andMorkinIsDead);
    print_tokens(0,0,0,0x1e,3);
    return 1;
  }
  
  // now check the Free's victory...
  if (g_garrisons[0x6].race != 0) {
    // we have taken Ushgarak!
    addToTokenList(ushHasFallen);
    addToTokenList(victoryToFree);
    print_tokens(0,0,0,0x1e,3);
    return 1;
  }
  if (icecrown) {
    // we have destroyed the ice crown
    addToTokenList(icecrownDestroyed);
    addToTokenList(victoryToFree);
    print_tokens(0,0,0,0x1e,3);
    return 1;
  }
  
  return 0;
}


void bringTheNight(void) {
  setupNightScreen();
  g_gameover = 0;
  if (checkVictoryDefeat()) {
    g_gameover = 1;
    return;
  }
  
  addToTokenList(nightHasFallen);
  g_saveArea.days++;
  numberToWords(g_saveArea.days);
  addPluralisedToken(0x6e);
  if (g_isPlural)
    addSingleToken(0xbd);   // {2 days} have
  else
    addSingleToken(0x8a);   // {1 day} has
  
  addToTokenList(passedSinceWarBegan);
  print_tokens(0,0,0,0x1e,3);
  
  calculateDDCitadels();
  resetLordStats();
  
  doDoomguardMoves();
  
  // finish off
  addToTokenList(dyawantDawn);
  print_tokens_pal(4); // green
  
}

/*used for wolves etc!!*/
void fightEnemy(void) {
  if (!g_selectedLord->alive)
    return;
  
  if (g_selectedLord->horse) {
    // check we don't lose it, 50/50 chance
    g_selectedLord->horse = randomish()&1;
    calculateGFX();
    
  }
  int them = randomish();
  int us = g_selectedLord->energy/2 - 0x40 + g_selectedLord->alive; 
  
  if (us >= them)
    return;
  
  // oops.
  g_selectedLord->alive = 0;
}



