
/*
* $Log: location.c,v $
* Revision 1.6  2004/01/27 20:19:13  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.5  2004/01/27 19:37:16  richq
* Underflow fix.
*
* Revision 1.4  2004/01/22 22:32:34  richq
* Icefear bug fix, the pointed to position was getting lost in the mix.
*
* Revision 1.3  2004/01/22 20:21:05  richq
* Saves updated location data. Special flag now set correctly.
*
* Revision 1.2  2004/01/22 20:05:15  richq
* workOutLocationDetails - code to work out who and what is at a particular position.
*                         
* Revision 1.1  2004/01/21 19:19:11  Default   
* Everything to do with locations goes in here.
*                                              
*/

#include "army.h"
#include "location.h"
#include "icefear.h"
#include "places.h"
#include "lords.h"
#include "features.h"

//location_t position;
locdat_t g_locdat;
location_t g_inFront;
location_t lookAtPos;

const int positionModifier[][2] = {
      {00, -1},      // N  ... x =  0, y = -1
      {01, -1},      // NE ... x =  1, y = -1
      {01, 00},      //  E ... x =  1, y =  0
      {01, 01},      // SE ... x =  1, y =  1
      {00, 01},      // S  ... x =  0, y =  1
      {-1, 01},      // SW ... x = -1, y =  1
      {-1, 00},      //  W ... x = -1, y =  0
      {-1, -1}       // NW ... x = -1, y = -1
};

void saveLocationData(void) {
  int x = g_locdat.x;
  int y = g_locdat.y-1;
  
  g_map[x + y*64] = g_locdat.terrain | (g_locdat.object<<4);
  g_areas[x + y*64] = g_locdat.area | g_locdat.domain | g_locdat.special;
  
}

void fillLocationData(locdat_t * data) {
  if ((s8)data->x < 0 || data->x > 0x3f || data->y <= 0 || data->y > 0x3d){
    data->terrain = WASTES;
    data->object = 0;
    data->area = 0;
    data->domain = 0;
    return;
  }
  int index = data->x+(data->y-1)*0x40;
  int mapd = g_map[index];
  int aread  = g_areas[index];
  
  data->terrain = mapd&0xf;
  data->object = (mapd>>4)&0xf;
  
  data->area = aread&0x3f;
  data->domain = aread&0x40;
  data->special = aread&0x80;

}

// project curr pos till we hit something
void lookingAt(location_t * pos) {
  // position is current position. 
  // use positionModifier to do this.
  
  int plains = 0;
  
  int xmod = positionModifier[pos->direction][0];
  int ymod = positionModifier[pos->direction][1];
  
  locdat_t loc;
  loc.x = pos->x;
  loc.y = pos->y;
  
  g_inFront.x = pos->x+xmod;
  g_inFront.y = pos->y+ymod;
  g_inFront.direction = pos->direction;
  
  do {
    loc.x += xmod;
    loc.y += ymod;
    fillLocationData(&loc);
    if (loc.terrain == 0xf) {
      plains++;
      if (plains == 3) {
        break;
      }
    }
  } while (loc.terrain == 0xf);
    
  lookAtPos.x = loc.x;
  lookAtPos.y = loc.y;
  lookAtPos.direction = pos->direction;
  
}


/*
 workOutLocationDetails Works out details at a location.
  fills in terrain, etc details in g_locdat
  calculates icefear
  sets up the tables of armies and lords
*/
void workOutLocationDetails(location_t * pos) {
  // e5ed
  g_locdat.x = pos->x;
  g_locdat.y = pos->y;
  fillLocationData(&g_locdat);
  // calculate icefear
  location_t  tmppos = *pos;
  calculate_icefear(&tmppos);
  
  // work out the energy modifier for free army riders
  g_riderEnergyMod = 0x40;
  if (g_locdat.terrain == MOUNTAIN)
    g_riderEnergyMod = 0x20;
  
  
  
  // reset the army details:
  g_doomdarkWarriors = 0;
  g_doomdarkRiders = 0;
  g_freeguard = 0;
  g_doomguard = 0;
  g_freeindex = 0;
  g_doomindex = 0;
  g_lordsHere = 0;
  
  int i;
  garrison_t * garrison;
  const place_t * place;
  // loop on special locations
  for (i = 0; i < 0x66; i++) {
    
    garrison = &g_garrisons[i];
    place = &specialPlacesTable[i];
    // keep track of where we got to...
    
    if (garrison->size) {
      // there is a garrison here
      if (place->x == g_locdat.x && place->y == g_locdat.y) {
        // aye, there's an army where we are!
        // add this garrison to battle table
        addGarrisonToBattleTable(i, garrison, &g_locdat);
        g_tmpGarrisonID = i;
        break;
      }
    }
  }
  
  // now loop on doomdarks wandering armies
  // e667
  for (i = 0; i < 128; i++) {
    if (g_regiments[i].size != 0) {
      if (g_regiments[i].x == g_locdat.x && 
          g_regiments[i].y == g_locdat.y)
      {
        // CALL e73f //Add reg to DoomDarks battle table
        addRegimentToBattle(i);
        
      }
      
    }
  }
  
  for (i = 0; i < 32; i++) {
    setSelectedLord(i);
    if (g_selectedLord->alive && !g_selectedLord->hidden &&
        g_locdat.x == g_selectedLord->location.x &&
        g_locdat.y == g_selectedLord->location.y) 
    {
       // call e75c // add character to battle table  
       addLordToBattle();
    }  
    
  }
  
  g_totalArmies = g_freeindex + g_doomindex;
  g_totalFree   = g_freeindex + g_lordsHere;
  
}

