/*
*  $Log: look.c,v $
*  Revision 1.6  2004/01/31 18:36:08  Default
*  Tried to make looking a bit smoother when left/right is held down.
*
*  Revision 1.5  2004/01/30 22:44:51  Default
*  pan left and right
*
*  Revision 1.4  2004/01/27 20:19:13  richq
*  The temp lord object has changed - it's now a pointer to the actual
*  table entry and gets rid of the need for all the copying
*  from temp to the table and back again.
*
*  Revision 1.3  2004/01/27 19:36:43  richq
*  Palette change better when Looking.
*
*  Revision 1.2  2004/01/25 21:23:44  richq
*  Movement split in 2 so that half of it can be called
*  when battles are started from the choice screen.
*
*  Revision 1.1  2004/01/22 22:25:16  richq
*  Code for looking about, moving forwards etc. Much of this moved from lords.c
*
*
*/
#include "gba.h"
#include "keypad.h"

#include "look.h"
#include "lords.h"
#include "savearea.h"
#include "objects.h"
#include "tokens.h"
#include "shield.h"
#include "army.h"
#include "think.h"
#include "features.h"
#include "gfx.h"
#include "choose.h"

int g_forwardsBlocked;


void drawDisplay(void) {
  
  // clear top screen
  clear_text();
  // clear bottom
  clear_objects();
  
  if (g_lordScreen != LOOK_SCREEN) {
    set_palette_time(g_selectedLord->time);
    clear_view();
  }
  
  // draw shield
  drawShield(g_selectedLordID);
  
  g_capsLock = 1;
  createLordTitle(g_selectedLord->title, g_selectedLord->firstname); 
  print_tokens(0, 0, 1, 0x17, 6);
  
  createHeStandsString(&g_selectedLord->location);
  print_tokens(1, 1, 0, 0x17, 5);
  
  
  drawPanorama(g_selectedLord->location.direction);
  // set horizon palette
  set_palette_time(g_selectedLord->time);
  
  // draw things in front of us
  drawInFront();
  
}


//6283
void look(void) {
  updateCurrentInfo();
  if (!g_selectedLord->alive) {
    // do think screen..
    g_lordScreen = LOOK_SCREEN;
    think();
  } else {
    // fc6a update display
    drawDisplay();
    updateCurrentInfo();
    g_lordScreen = LOOK_SCREEN;
  }
}




void applyMovementCost(void) {
  // 631b
  int cost = 1;
  if (g_selectedLordID != FARFLAME) {
    // straight = 2, diagonal = 3
    cost = 2 + (g_selectedLord->location.direction&1);
    if (!g_selectedLord->horse)
      cost *= 2;
    // now adjust for terrain...
    switch (g_locdat.terrain) {
      case DOWNS:     cost++; 
                      break;
      case MOUNTAIN: cost+=4; 
                      break;
      case FOREST:    cost+=(g_selectedLord->race == 2)?0:3;    // forest impedes everyone but the Fey
                      break;
      // everything else has no effect.
      default:        break;
    }
  }
  int newval = g_selectedLord->time - cost;
  g_selectedLord->time = newval<0?0:newval;
  
  newval = g_selectedLord->energy - cost;
  g_selectedLord->energy = newval<0?0:newval;
  
  newval = g_selectedLord->riders_energy - cost;
  g_selectedLord->riders_energy = newval<0?0:newval;
  
  newval = g_selectedLord->warriors_energy - cost;
  g_selectedLord->warriors_energy = newval<0?0:newval;
  
  
}

// for the current state of the world...
// set the tile for the g_locdat to armies or not
void setTileToArmy(int armies) {
  int newTile = ARMY;
  if (armies == 0) {
    newTile = PLAIN;
  }
  
  if (g_locdat.terrain >= ARMY) {
    g_locdat.terrain = newTile;
    saveLocationData();
  }
}


void updateArmyTiles(location_t * oldLocation, location_t * newLocation) {
  
  // work out loc details for each
  workOutLocationDetails(oldLocation);
  setTileToArmy(g_totalArmies);
  workOutLocationDetails(newLocation);
  setTileToArmy(g_totalArmies);
  
}

// 6303
void doMove(void) {
  location_t oldLocation = g_selectedLord->location;
  location_t newLocation = g_inFront;
  g_selectedLord->location = g_inFront;
  // update map with army tiles...
  updateArmyTiles(&oldLocation, &newLocation);
  
  // now the "world state" is that of our new location
  setSelectedLord(g_saveArea.selectedLord);
  
  // calculate movement cost and apply it
  applyMovementCost();
  look();
}


void moveForwards(void) {
  updateCurrentInfo();
  if ( g_forwardsBlocked || 
      !g_selectedLord->alive || 
      !g_selectedLord->energy || 
      !g_selectedLord->time ||
      (g_selectedLord->time != 0x10 /*dawn*/ && g_doomindex)
  ) {
    think();
    return;
  }
  if (g_selectedLord->hidden ||
      (g_locdat.object > OBJECT_NOTHING && g_locdat.object < OBJECT_WILD_HORSES)
  ) {
    choose();
    return;
  }
  g_panning = 0;
  doMove();
}

void rotateLeft(void) {
  while (KeyPressed(KEY_LEFT) ) {
    setSelectedLord(g_saveArea.selectedLord);
    g_selectedLord->location.direction--;
    if ((s8)g_selectedLord->location.direction < 0)
      g_selectedLord->location.direction = 7;
    
    if (g_panningOn)
      g_panning = PAN_LEFT;
    
        
    look();
    if (!g_panningOn)
      break;
    
    UpdateKeys();
  }
  
}
void rotateRight(void) {
  while (KeyPressed(KEY_RIGHT) ) {
    setSelectedLord(g_saveArea.selectedLord);
    g_selectedLord->location.direction++;
    if (g_selectedLord->location.direction > 7)
      g_selectedLord->location.direction = 0;
    
    if (g_panningOn) {
      g_panning = PAN_RIGHT;
    }
    look();
    if (!g_panningOn)
      break;
    UpdateKeys();
  }
}

/* set the value for if we can move forwards or not
makes use of loads of globals!
*/
void setForwardsBlocked(void) {
  g_forwardsBlocked = (g_locdat.terrain == WASTES) || 
                      (g_freeindex > 0x1d) ||
                      (g_doomindex);
}

