/*
* $Log: menu.c,v $
* Revision 1.2  2004/01/31 15:32:51  Default
* fixed menu for > 18 items
*
* Revision 1.1  2004/01/27 19:41:30  richq
* Menu stuff. Most of this is not very portable... Uses a sprite for the
* highlighter and uses the tile map to find where the next option is
* (looks down columns for the # character).
*
* 
*/

#include "gba.h"
#include "gfx.h"

#include "menu.h"

// where the sprite is in oam
#define HIGHLIGHT_SPRITE 0
#define HIGHLIGHT_PRIORITY 1
#define HIGHLIGHT_X 1

int g_highlightOn;

void createHighlightTiles(void) {
  
  u8 tileBuffer[8*8];
  int i, j;
  for (i = 0; i < 8; i++)
    for (j = 0; j < 8; j++) 
      tileBuffer[i+j*8] = (i==0||j==7)?0:2; // blue
  
  drawImage(tileBuffer,0, 0, HIGHLIGHT_SPRITE);
  // set between text and bg..
  init_sprite64(HIGHLIGHT_SPRITE, HIGHLIGHT_PRIORITY);
  g_highlightOn = 1;
  
  // now alpha blend between the highlighter and the text layer
  REG_BLDMOD = BLDMOD_FIRST_BG2|BLDMOD_FIRST_BG1 | BLDMOD_SECOND_OBJ | BLDMOD_MODE(1);
  REG_COLEV = (12)|((4)<<8);
  
}

void moveHighlighter(int x, int y) {
  // the highlight sprite...
  if (!g_highlightOn)
    createHighlightTiles();
    
  move_sprite(HIGHLIGHT_SPRITE, x*8, y*8);
  
}

void clearHighlighter(void) {
  g_highlightOn = 0;
  remove_sprite(HIGHLIGHT_SPRITE);
  CopyOAM();
  REG_BLDMOD = 0;
}

void getChoiceXY(int choice, int *x, int *y) {
  
  // look for # characters in the text map at x = 1
  int choicey = 0;
  int choicex = 0x1;

  choice++;
  int found = 0;
  int tileID = (FONT_TILE - 0x20 ) + 0x2000 * bg_text.charBaseBlock;
  
  while (choice > found && choicey <= 20) {
    if ( (bg_text.mapData[choicex+choicey*32] ==  (tileID+ASCII_BOX_OK)) || 
         (bg_text.mapData[choicex+choicey*32] ==  (tileID+ASCII_BOX_DEAD)) || 
         (bg_text.mapData[choicex+choicey*32] ==  (tileID+ASCII_BOX_TIRED)) || 
         (bg_text.mapData[choicex+choicey*32] ==  (tileID+ASCII_BOX_NIGHT))
       ) {
      found++;
    }
    choicey++;
    if (choicey == 0x13 && choicex == 1) {
      choicex = 0x10;
      choicey = 0;
    }
    
  }
  *y = choicey - 1;
  *x = choicex;
  
}

void selectOption(int i) {
  int x, y;
  getChoiceXY(i, &x, &y);
  moveHighlighter(x, y);
}


