/*
* $Log: savearea.c,v $
* Revision 1.5  2004/01/31 15:34:08  Default
* Added the option to switch off the smooth panning.
*
* Revision 1.4  2004/01/30 23:01:09  Default
* name change of global lords table
*
* Revision 1.3  2004/01/26 20:33:32  richq
* Saving - everything is in this module, from menus to controls
* to the fiddly byte-by-byte saving of the structs.
*
* Revision 1.2  2004/01/22 19:58:13  richq
* Got rid of malloc'd save area. I thought the save area was bigger, but
* most of it is actually stored in other tables.
*                                                              
* Revision 1.1  2004/01/21 19:27:42  Default                                        
* Contains a struct for bits of data to save. When saving is implemented, that is :)
*                                                                                   
*/

#include "stdlib.h"
#include "keypad.h"
#include "saving.h"

#include "select.h"
#include "places.h"
#include "army.h"
#include "lords.h"
#include "savearea.h"
#include "look.h"
#include "tokens.h"
#include "gfx.h"
#include "think.h"
#include "menu.h"

void wait_vsync_int(void);
//
//// these may well be wrong... see menu.h for what I mean
//extern void getMenuXY(int item, int *x, int *y);
//extern void moveHighlighter(int x, int y);
//extern void clearHighlighter();

savearea_t g_saveArea;
int l_highlightSlot;
int l_maxHighlight;

void init_savearea(void) {
//  if (g_saveArea == 0)
//    g_saveArea = (savearea_t *) malloc(sizeof(savearea_t));
    
  g_saveArea.days = 0;
  g_saveArea.selectedLord = 0;
  
}

// individual size of save data (is slightly bigger than necessary):
#define SAVE_SIZE 5400
// slots
#define SAVE_SLOTS 3
#define SAVE_OPTIONS 5
// skip first few bytes, as they are dodgy (?)
#define SAVE_OFFSET 3
// header position is offset + 3 byte "key"
#define SAVE_HEADER_POS (SAVE_OFFSET+3)
// the key to look for - so we know this is our save data
const char * const saveKey = "LOM";

// header. Add to this later
#define HEADER_SIZE 1
typedef struct {
  u8 isSaved;   // bit flag for which slots are filled
  // time played? this would have to be per game
//  u16 hours;
//  u8 minutes;

}
header_t;

// start of save data
#define SAVE_DATA_POS (SAVE_HEADER_POS + HEADER_SIZE)

/*
Saving: have a save screen
Save area is 6290 bytes big originally. Data saved is:

days - days since the start of the game
char - current selected character
      size = 4
garrisons
    102 of 'em, 2 bytes each = 204 bytes
(special places      <-- not needed as is const, prevents saving 448 bytes!)
regiments   128, 4 bytes per entry = 512 bytes
characters  32, 32 bytes each = 1024 bytes      (could get this down to 768 bytes)
terrain map (not areas) 64*61 = 3904

all the data packed comes to 5648 bytes

*/

//const u8 saveString = {0xfc, };
const u8 gameSaved[] = {0xfc,0x84,0xfc,0x82,0xfe,0x64,0xfe,0x21,0xff };
const u8 saveGame[] = {0xfc,0x82,0xfc,0x84,0xff };
const u8 loadGame[] = {0xfc,0xaa,0xfc,0x84,0xff };

// Choose save to free up (no delete token!)

// replace "verify" (8f) with "delete"
const u8 deleteGame[] = {0xfc,0x8f,0xfc,0x84,0xff};
const u8 gameDeleted[] = {0xfc,0x84,0xfc,0x8f,0xfe,0x64,0xfe,0x21,0xff};
const u8 smoothLooking[] = {0xfc, 0x93, 0xfc, 0x86, 0xfd, 0x98, 0xff};


// is our save key in sram?
int hasSaveKey() {
  char key;
  int i;
  for (i = 0; i < 3; i++) {
    key = LoadByte(SAVE_OFFSET + i);
    if (key != saveKey[i])
      return 0;
  }
  return 1;
}


int loadGameDays(int gameid) {
  int offset = SAVE_DATA_POS + SAVE_SIZE*gameid;

  int days = LoadByte(offset++);
  days += 256*LoadByte(offset++);
  return days;
}


void saveHeader(const header_t * h) {
  int i;
  for (i = 0; i < 3; i++)
    SaveByte(SAVE_OFFSET + i, saveKey[i]);

  SaveByte(SAVE_HEADER_POS, h->isSaved);
}

void loadHeader(header_t * h) {
  int i = 0;
  h->isSaved = LoadByte(SAVE_HEADER_POS + i);
  i++;
}

// []1 Day one
// []2 None
//  .. etc ..
void addSlotEntry(int days, int id) {
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('1'+id);
  if (days >= 0) {
    addNameToken(0x6e); // Day
    numberToWords(days+1);  // one, two, etc
  } else {
    addNameToken(0xcc); // None (a7 would be New)
  }
}


// shows list of saves already present
void showCurrentList() {
  int i;
  addNewlineToken();
  // load the list of saves
  header_t header;
  if (!hasSaveKey()) {
    // no saved games, create save game header
    header.isSaved = 0;
    saveHeader(&header);
  }

  // OK, we have a header, now see which games are here...
  loadHeader(&header);
  u8 bitmask;
  int days;
  for (i = 0; i < SAVE_SLOTS; i++) {
    bitmask = i==0?1:1<<i;
    days = -1;
    if (header.isSaved & bitmask) {
      // add a save game name
      days = loadGameDays(i);
    }
    addSlotEntry(days, i);  // Day one, etc
  }

}

void doSavedInfoScreen(void) {
  clear_text();
  print_tokens(2,2,0,0x1e,2);
  showCurrentList();
  l_maxHighlight = SAVE_SLOTS;
  print_tokens_pal(2);
}

int (*actionPerformed)(void);
int (*actionCancelled)(void);

// save data in slot l_highlightSlot
int saveData(void) {
  int i;
  header_t header;
  loadHeader(&header);
  header.isSaved |= (l_highlightSlot==0)?1:(1<<l_highlightSlot);
  saveHeader(&header);

  // now save the data...
  int index = SAVE_DATA_POS + SAVE_SIZE*l_highlightSlot;
  // days...
  SaveByte(index++, g_saveArea.days&0xff);
  SaveByte(index++, (g_saveArea.days/256)&0xff);
  // who
  SaveByte(index++, g_saveArea.selectedLord);
  // "4" bytes, say
  
  // garrisons..
  u8 theByte;
  for (i = 0; i < 102; i++) {
    theByte = (g_garrisons[i].type*4) | (g_garrisons[i].race);
    SaveByte(index++, theByte);
    SaveByte(index++, g_garrisons[i].size);
  } // 204 bytes

  // regiments..
  for (i = 0; i < 128; i++) {
    theByte = g_regiments[i].orders | g_regiments[i].x;
    SaveByte(index++, theByte);
    theByte = g_regiments[i].type | g_regiments[i].y;
    SaveByte(index++, theByte);
    SaveByte(index++, g_regiments[i].size);
    SaveByte(index++, g_regiments[i].id);
  } // 512
  // characters
  for (i = 0; i < 32; i++) {
    SaveByte(index++, g_lordTable[i].location.x);
    SaveByte(index++, g_lordTable[i].location.y);
    SaveByte(index++, g_lordTable[i].location.direction);
    SaveByte(index++, g_lordTable[i].time);
    SaveByte(index++, g_lordTable[i].moonring);
    SaveByte(index++, g_lordTable[i].graphic);
    SaveByte(index++, g_lordTable[i].riders);
    SaveByte(index++, g_lordTable[i].riders_energy);
    SaveByte(index++, g_lordTable[i].warriors);
    SaveByte(index++, g_lordTable[i].warriors_energy);
    SaveByte(index++, g_lordTable[i].battle_area);
    SaveByte(index++, g_lordTable[i].riders_lost);
    SaveByte(index++, g_lordTable[i].warriors_lost);
    SaveByte(index++, g_lordTable[i].killed);
    SaveByte(index++, g_lordTable[i].riders_killed);
    SaveByte(index++, g_lordTable[i].warriors_killed);
    SaveByte(index++, g_lordTable[i].battle_status);
    SaveByte(index++, g_lordTable[i].alive);
    SaveByte(index++, g_lordTable[i].energy);
    SaveByte(index++, g_lordTable[i].courage);
    SaveByte(index++, g_lordTable[i].hidden);
    SaveByte(index++, g_lordTable[i].horse);
    SaveByte(index++, g_lordTable[i].object);
    SaveByte(index++, g_lordTable[i].deathby);
  } // 24*32 = 768
  
  // terrain map
  for (i = 0; i < 64*61; i++) {
    SaveByte(index++, g_map[i]);
  } // 3904
  
  
  addToTokenList(gameSaved);
  doSavedInfoScreen();
  return 0;
}

// load from slot l_highlightSlot
int loadData(void) {
  int i;
  header_t header;
  loadHeader(&header);
  int bitmask = ((l_highlightSlot==0)?1:(1<<l_highlightSlot));
  if (!(header.isSaved & bitmask)) {
    // this slot not available
    return 0;
  }
  // now load the data...
  int index = SAVE_DATA_POS + SAVE_SIZE*l_highlightSlot;
  
  // days...
  g_saveArea.days = LoadByte(index++);
  g_saveArea.days |= 256 * LoadByte(index++);
  // who
  g_saveArea.selectedLord = LoadByte(index++);
  // "4" bytes
  
  // garrisons..
  u8 theByte;
  for (i = 0; i < 102; i++) {
    theByte = LoadByte(index++);
    g_garrisons[i].type = (theByte&4)/4;
    g_garrisons[i].race = (theByte&3);
    g_garrisons[i].size = LoadByte(index++);
    
  } // 204 bytes

  // regiments..
  for (i = 0; i < 128; i++) {
    theByte = LoadByte(index++);
    g_regiments[i].orders = theByte & 0xc0;
    g_regiments[i].x = theByte & 0x3f;
    
    theByte = LoadByte(index++);
    g_regiments[i].type = theByte & 0x80;
    g_regiments[i].y = theByte & 0x3f;
    
    g_regiments[i].size = LoadByte(index++);
    g_regiments[i].id = LoadByte(index++);
  } // 512
  // characters
  for (i = 0; i < 32; i++) {
    g_lordTable[i].location.x = LoadByte(index++);
    g_lordTable[i].location.y = LoadByte(index++);
    g_lordTable[i].location.direction = LoadByte(index++);
    g_lordTable[i].time = LoadByte(index++);
    g_lordTable[i].moonring = LoadByte(index++);
    g_lordTable[i].graphic = LoadByte(index++);
    g_lordTable[i].riders = LoadByte(index++);
    g_lordTable[i].riders_energy = LoadByte(index++);
    g_lordTable[i].warriors = LoadByte(index++);
    g_lordTable[i].warriors_energy = LoadByte(index++);
    g_lordTable[i].battle_area = LoadByte(index++);
    g_lordTable[i].riders_lost = LoadByte(index++);
    g_lordTable[i].warriors_lost = LoadByte(index++);
    g_lordTable[i].killed = LoadByte(index++);
    g_lordTable[i].riders_killed = LoadByte(index++);
    g_lordTable[i].warriors_killed = LoadByte(index++);
    g_lordTable[i].battle_status = LoadByte(index++);
    g_lordTable[i].alive = LoadByte(index++);
    g_lordTable[i].energy = LoadByte(index++);
    g_lordTable[i].courage = LoadByte(index++);
    g_lordTable[i].hidden = LoadByte(index++);
    g_lordTable[i].horse = LoadByte(index++);
    g_lordTable[i].object = LoadByte(index++);
    g_lordTable[i].deathby = LoadByte(index++);
  } // 24*32 = 768
  
  // terrain map
  for (i = 0; i < 64*61; i++) {
    g_map[i] = LoadByte(index++);
  } // 3904
  
  return 1;
}

int deleteData(void) {
  header_t header;
  loadHeader(&header);
  int bitmask = ((l_highlightSlot==0)?1:(1<<l_highlightSlot));
  if (!(header.isSaved & bitmask)) {
    // this slot not available
    return 0;
  }
  else {
    header.isSaved &= ~bitmask;
    saveHeader(&header);
    
    addToTokenList(gameDeleted);
    doSavedInfoScreen();
    return 0;
  }
}

void waitKeys() {

  UpdateKeys();
  u16 delay = 10;
  while (1) {
    
    wait_vsync_int();
    UpdateKeys();
    
    if (KeyPressedNoBounce(KEY_UP,delay)) {
      if (l_highlightSlot) {
        l_highlightSlot--;
        selectOption(l_highlightSlot);
      }

    }

    if (KeyPressedNoBounce(KEY_DOWN,delay)) {
      if (l_highlightSlot < l_maxHighlight-1) {
        l_highlightSlot++;
        selectOption(l_highlightSlot);
      }
    }

    if (KeyPressedNoBounce(KEY_A,delay)) {
      if (actionPerformed()) {
        clearHighlighter();
        look();
        WaitKeyReleased(KEY_A);
        return;
      }
      WaitKeyReleased(KEY_A);
    }
    if (KeyPressedNoBounce(KEY_B,delay)) {
      // cancel..
      if (actionCancelled()) {
        look();
        WaitKeyReleased(KEY_B);
        // wait for let go?
        return;
      } // else carry on checking keys
      WaitKeyReleased(KEY_B);
    }
  }
}


int doMainSaveMenu(void);

void save(void) {
  addToTokenList(saveGame);
  doSavedInfoScreen();
  actionPerformed = saveData;
  actionCancelled = doMainSaveMenu;
  l_highlightSlot = 0;
  selectOption(l_highlightSlot);
}

void load(void) {
  addToTokenList(loadGame);
  doSavedInfoScreen();
  actionPerformed = loadData;
  actionCancelled = doMainSaveMenu;
  l_highlightSlot = 0;
  selectOption(l_highlightSlot);
}

void deleteSave(void) {
  addToTokenList(deleteGame);
  doSavedInfoScreen();
  actionPerformed = deleteData;
  actionCancelled = doMainSaveMenu;
  l_highlightSlot = 0;
  selectOption(l_highlightSlot);
}

int cancelMenu(void) {
  clearHighlighter();
  return 1;
}

void toggleSmoothLook() {
  g_panningOn = !g_panningOn;
  int tmpSel = l_highlightSlot;
  doMainSaveMenu();
  l_highlightSlot = tmpSel;
  selectOption(l_highlightSlot);
}


int makeSaveSelection(void) {
  
  switch (l_highlightSlot) {
    case 0: save(); break;
    case 1: load(); break;
    case 2: deleteSave(); break;
    case 3: toggleSmoothLook(); break;
    default : return 1; 
  }
  return 0;
}

int doMainSaveMenu(void) {
  clearHighlighter();
  
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('1');
  addToTokenList(saveGame);
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('2');
  addToTokenList(loadGame);
  
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('3');
  addToTokenList(deleteGame);
  
  addNewlineToken();
  if (g_panningOn) {
    addLiteralToken(ASCII_BOX_NIGHT);
  } else {
    addLiteralToken(ASCII_BOX_OK);
  }
  addLiteralToken('4');
  addToTokenList(smoothLooking);
  
  addNewlineToken();
  addLiteralToken(ASCII_BOX_OK);
  addLiteralToken('5');
  addNameToken(0x92); // Continue
  
  clear_text();
  
  print_tokens(2,2,0,0x1e,2);
  // highlight one, then do the menu
  l_highlightSlot = 0;
  actionPerformed = makeSaveSelection;
  actionCancelled = cancelMenu;

  selectOption(l_highlightSlot);
  l_maxHighlight = SAVE_OPTIONS;
  return 0;
}



// call this from START or something
void showSaveMenu(void) {
  // clear + set palette
  setupInfoScreen();
  doMainSaveMenu();
  g_lordScreen = MENU_SCREEN;
  // loop on keys - pressing A or B calls 
  // actionPerformed (A) or actionCancelled (B)
  // if either returns true, then control returns here
  // until then it keeps looping
  waitKeys();
}


/*
Bytes not needed from lord data

  u8 firstname;           //BYTE 04 - first name token
  u8 title;               //BYTE 05

  u8 strength;
  u8 cowardess;
  u8 recruiting_key;      //BYTE 23 the character who is doing the recruiting, recruiting_key is
  u8 recruited_by;        //BYTE 24 anded with the recruited_by from the character he wished to recruit and if it is not zero
                          //        then recruiting can take place.
  u8 unused;              //BYTE 26
  u8 race;                //BYTE 28 - see race table

} lord_t;

So, these are needed:
  location_t location.x;
  location.y;
  location.direction;
  u8 time;                //BYTE 03 - time of day
  u8 moonring;            //BYTE
  u8 graphic;             //BYTE 07
  u8 riders;              //BYTE 08 - total riders / 5
  u8 riders_energy;       //BYTE 09 - riders energy
  u8 warriors;            //BYTE 10 - total warriors / 5
  u8 warriors_energy;     //BYTE 11 - warriors energy
  u8 battle_area;         //BYTE 12 - area currently in battle in
  u8 riders_lost;         //BYTE 13 - number of riders lost in battle
  u8 warriors_lost;       //BYTE 14 - number of warriors lost in battle
  u8 killed;              //BYTE 15 - number of the enemy killed by the character in battle
  u8 riders_killed;       //BYTE 16 - enemy killed by riders
  u8 warriors_killed;     //BYTE 17 - enemy killed by warriors
  u8 battle_status;       //BYTE 18 - 0xff no batle, 0 - battle continues, ?? - who won
  u8 alive;               //BYTE 19 - 0 or 1
  u8 energy;              //BYTE 20 - current energy
  u8 courage;             //BYTE 25
  u8 hidden;              //BYTE 27 - 0 or 1
  u8 horse;               //BYTE 29 - 0 or 1
  u8 object;              //BYTE 30 - object carrying
  u8 deathby;             //BYTE 31 , 0 - in battle, otherwise the value is the object that killed him

*/

/*
new token data, with delete replacing verify is:

0x84, 0xa2, 0x92, 0xd8, 0xa3, 0xe8, 0xc9, 0xc4,
0xcc, 0x61, 0xef, 0xb5, 0xd2, 0xde, 0x73, 0xa7,
0x25, 0x29, 0xdf, 0x3d, 0x87, 0x3d, 0x99, 0x1a,
0x5a, 0x4f, 0xae, 0x44, 0x51, 0x61, 0x6f, 0xd2,
0x42, 0x4a, 0x20, 0xe6, 0x96, 0x24, 0x89, 0xa9,
0x47, 0x86, 0x65, 0x48, 0x09, 0x14, 0x15, 0xf2,
0x5e, 0x43, 0xa4, 0x4c, 0x18, 0x92, 0x44, 0xf6,
0xc9, 0x13, 0x28, 0x4a, 0x75, 0xa2, 0x13, 0x64,
0x58, 0xa7, 0x1e, 0xf9, 0xa4, 0x23, 0xeb, 0xc9,
0x43, 0xdf, 0x6b, 0xef, 0x98, 0x77, 0x64, 0x6a,
0x87, 0x3c, 0x22, 0xc3, 0x4a, 0xe7, 0xc9, 0x23,
0x03, 0x45, 0xc9, 0x06, 0x56, 0x28, 0x7a, 0xf2,
0x8c, 0x27, 0x1f, 0x45, 0x28, 0x4c, 0x94, 0x5a,
0x90, 0x27, 0x51, 0x24, 0x13, 0x43, 0x13, 0x05,
0xf2, 0xee, 0x34, 0x82, 0xbd, 0x47, 0x0e, 0x45,
0x32, 0x30, 0x86, 0xe8, 0x93, 0x4b, 0x16, 0x57,
0x4e, 0x90, 0x04, 0xb5, 0x44, 0x51, 0x90, 0xa7,
0xb4, 0xf7, 0x1c, 0x4a, 0xb3, 0xc8, 0x40, 0x51,
0x9a, 0x28, 0xb5, 0x22, 0x13, 0x3b, 0x8f, 0xc8,
0x20, 0x57, 0x4a, 0x13, 0xb5, 0xc0, 0x42, 0x36,
0x42, 0x26, 0x8a, 0xd0, 0x4c, 0xac, 0xb8, 0x51,
0x4e, 0x61, 0xc5, 0x25, 0x76, 0xe4, 0x93, 0x81,
0x22, 0xd4, 0x9e, 0x74, 0xc9, 0x9d, 0x44, 0x51,
0x6a, 0x32, 0x50, 0x84, 0xc6, 0x93, 0x85, 0x85,
0x9c, 0x64, 0xa0, 0x48, 0x3e, 0xe9, 0x10, 0x7b,
0xf2, 0xc8, 0xd4, 0x08, 0x7b, 0x12, 0x99, 0x40,
0x64, 0xa0, 0xc8, 0x10, 0xf9, 0x9e, 0x33, 0x47,
0x06, 0x79, 0x0e, 0x91, 0x25, 0xb4, 0x89, 0x12,
0x45, 0xf2, 0x49, 0x97, 0x2e, 0x4a, 0x13, 0x16,
0x39, 0x15, 0x0e, 0xea, 0xc9, 0x95, 0x28, 0x3a,
0xa8, 0xc8, 0x40, 0x91, 0x59, 0xb5, 0x05, 0x99,
0x5a, 0x90, 0x61, 0x25, 0x8a, 0x0a, 0x0d, 0xf2,
0x98, 0x4a, 0x83, 0x3c, 0x28, 0x50, 0xf4, 0x64,
0x62, 0x88, 0xc8, 0x72, 0x64, 0x6a, 0x85, 0x84,
0xeb, 0x0c, 0x95, 0x4f, 0xba, 0x32, 0xde, 0x6b,
0xe3, 0x99, 0xd4, 0x5e, 0x75, 0x34, 0x24, 0x87,
0x46, 0xa2, 0x81, 0x14, 0x76, 0xcc, 0x93, 0x65,
0x52, 0x83, 0x8a, 0x3b, 0xc5, 0xd0, 0x77, 0x8b,
0x44, 0x36, 0x31, 0x16, 0x4e, 0x29, 0xe4, 0x5d,
0x57, 0x56, 0x29, 0x30, 0x4d, 0xf7, 0x2e, 0x0a,
0x8c, 0x15, 0x16, 0x56, 0x31, 0x37, 0x4c, 0x5a,
0x8a, 0xac, 0xc9, 0x15, 0x96, 0x28, 0x3a, 0x23,
0xd8, 0x22, 0x5d, 0x09, 0xb2, 0x65, 0x03, 0x59,
0x48, 0xee, 0x18, 0xf9, 0x74, 0x71, 0x87, 0xbc,
0x16, 0x92, 0x13, 0x61, 0x24, 0x37, 0x50, 0x31,
0x93, 0x06, 0x47, 0x8c, 0x7b, 0x92, 0x22, 0x33,
0x5f, 0xa5, 0xa8, 0x94, 0x30, 0x69, 0xb9, 0x65,
0x52, 0xc4, 0xde, 0x5b, 0xc9, 0x9d, 0x41, 0xdf,
0x08, 0x74, 0x3c, 0x77, 0x44, 0x99, 0x89, 0x94,
0xc2, 0xca, 0x9c, 0x85, 0xa2, 0xe0, 0x0a, 0x7a,
0x55, 0x92, 0xe7, 0x0a, 0xa1, 0xf7, 0x19, 0x8a,
0x64, 0x29, 0xc5, 0xbc, 0x2a, 0x8b, 0x49, 0xb6,
0x90, 0x99, 0xb0, 0x99, 0xc5, 0x16, 0x67, 0x4a,
0x3a, 0x88, 0x16, 0x97, 0x5c, 0x21, 0xb4, 0xb8,
0x23, 0x4b, 0x0b, 0xc9, 0x11, 0x12, 0xf2, 0x48,
0x74, 0xa4, 0x69, 0x5c, 0x3a, 0x88, 0x1a, 0x56,
0xf1, 0x93, 0xa7, 0x3d, 0xb9, 0x92, 0x43, 0xe3,
0x49, 0x56, 0x28, 0x4a, 0xf2, 0x49, 0x8a, 0xe4,
0x73, 0xc8, 0x84, 0xbb, 0x64, 0x72, 0xc9, 0x04,
0x69, 0x58, 0x68, 0x05, 0xad, 0x27, 0x93, 0x2b,
0xac, 0xb0, 0x27, 0x09, 0x31, 0x25, 0x97, 0x9b,
0x66, 0x49, 0x73, 0x55, 0xb6, 0x64, 0x72, 0x87,
0xc8, 0x70, 0x9e, 0x5b, 0xed, 0x3d, 0x07, 0x65,
0x72, 0xa3, 0x18, 0x28, 0xcb, 0x9c, 0x67, 0xa0,
0xf7, 0x66, 0x21, 0xab, 0xe4, 0x32, 0x83, 0x2d,
0xe7, 0x51, 0x54, 0xe4, 0x2c, 0x27, 0xb4, 0x62,
0x26, 0x0d, 0x92, 0x16, 0x1a, 0x60, 0x21, 0xc1,
0x11, 0x53, 0x1c, 0x2d, 0xd2, 0x88, 0x92, 0xdc,
0x21, 0x28, 0xcc, 0x22, 0x4a, 0x71, 0x83, 0x3c,
0x92, 0x5f, 0x2d, 0x37, 0x38, 0x7a, 0x48, 0x61,
0x85, 0x16, 0x72, 0xc3, 0x5c, 0xb3, 0x8e, 0x51,
0xe6, 0x34, 0x95, 0xa5, 0xf1, 0x1c, 0x4d, 0xae,
0x96, 0x63, 0x42, 0x62, 0x85, 0x98, 0x70, 0x42,
0x72, 0x23, 0x38, 0xf2, 0x18, 0x21, 0x54, 0x66,
0x92, 0xdc, 0x31, 0xe8, 0xc9, 0x59, 0xca, 0x7d,
0xcc, 0x90, 0x34, 0x8a, 0xa1, 0xf2, 0x31, 0x56,
0x6e, 0x62, 0xc4, 0x9c, 0x1a, 0x24, 0x29, 0x46,
0x96, 0x52, 0x48, 0x90, 0x2b, 0xb5, 0x44, 0x5c,
0x3a, 0x88, 0xa2, 0x5b, 0xd8, 0x78, 0xad, 0x14,
0x16, 0x1c, 0x29, 0xad, 0x4e, 0x6a, 0x4c, 0x92,
0x93, 0x22, 0x51, 0xe8, 0x2d, 0xc5, 0x11, 0x57,
0xee, 0x78, 0xb2, 0x98, 0x12, 0x64, 0x61, 0x2f,
0x90, 0x63, 0x02, 0x63, 0xc5, 0x15, 0xf3, 0x2a,
0x3b, 0x41, 0xc8, 0x17, 0x08, 0x09, 0xb2, 0x18,
0xf3, 0xaa, 0x23, 0xc8, 0x3d, 0x8a, 0x92, 0x3b,
0x67, 0x96, 0x20, 0x07, 0x2a, 0x2d, 0xb8, 0x7c,
0x0a, 0x2b, 0xb6, 0xb8, 0x43, 0x6f, 0x61, 0xb6,
0x14, 0x8a, 0x92, 0x24, 0xe6, 0x49, 0x62, 0x92,
0x29, 0xb4, 0x14, 0x77, 0xe8, 0x2d, 0x8e, 0x66,
0x84, 0xaa, 0x23, 0xb2, 0x90, 0x44, 0xd1, 0xab,
0x33, 0x38, 0x32, 0xdc, 0x2b, 0x28, 0xd8, 0x52,
0x60, 0x98, 0x93, 0xdc, 0x20, 0xd3, 0x7c, 0x6c,
0xd2, 0x34, 0x8a, 0x11, 0xe5, 0x04, 0x67, 0x66,
0x72, 0xa3, 0x1c, 0x11, 0x28, 0x65, 0xc5, 0xc8,
0x44, 0x8a, 0x44, 0x37, 0x48, 0x99, 0x9e, 0x34,
0x81, 0x3d, 0x57, 0xca, 0x64, 0xe5, 0x9a, 0xe2,
0x4a, 0xcb, 0x87, 0xa2, 0xe4, 0xd6, 0x44, 0x36,
0x0d, 0xfa, 0x64, 0x2e, 0xb7, 0x38, 0x5a, 0xdc,
0x73, 0x85, 0xb4, 0xe2, 0x88, 0xa8, 0x94, 0xa0,
0x34, 0x49, 0x60, 0x0c, 0x55, 0x4a, 0x8b, 0x64,
0xb9, 0xd8, 0x22, 0x73, 0x9a, 0x2c, 0x1d, 0x44,
0x59, 0x36, 0xb1, 0x26, 0x5a, 0x0c, 0x4d, 0xb2,
0x10, 0x96, 0x9c, 0x4d, 0xe7, 0xc9, 0x40, 0x0b,
0x41, 0xe3, 0xb5, 0x16, 0x1c, 0x39, 0x23, 0x38,
0xf7, 0x68, 0x3a, 0x35, 0x91, 0xe0, 0x46, 0x31,
0xf6, 0xa5, 0x51, 0xcc, 0x08, 0x8c, 0x4d, 0x34,
0x51, 0x61, 0xb4, 0x48, 0x25, 0x8b, 0x91, 0x65,
0xa2, 0x42, 0xcc, 0x45, 0x69, 0x20, 0x43, 0x64,
0x48, 0xae, 0x98, 0xc4, 0xd8, 0xb1, 0xe9, 0xbc,
0x2a, 0x0b, 0x4b, 0xa6, 0x94, 0x9b, 0x28, 0x2a,
0x28, 0x38, 0x72, 0x84, 0x4c, 0xee, 0x4c, 0x33,
0xef, 0x93, 0xe4, 0x4c, 0x16, 0x72, 0x91, 0xe7,
0x06, 0x5a, 0xe4, 0x24, 0xa3, 0x42, 0x33, 0xe4,
0xbb, 0x0e, 0x91, 0x32, 0xa9, 0x7c, 0xd9, 0xb0,
0x12, 0xc8, 0x24, 0x23, 0xb8, 0x82, 0x12, 0x29,
0x07, 0x25, 0x42, 0x8a, 0x2b, 0xb3, 0x94, 0x75,
0x56, 0x62, 0xac, 0x90, 0x13, 0x8c, 0x0c, 0x89,
0x14, 0xf1, 0x18, 0x29, 0xe3, 0x31, 0x52, 0x5a,
0x62, 0x04, 0xc9, 0x32, 0x64, 0x4d, 0xd4, 0x98,
0x74, 0x10, 0x3d, 0xf7, 0x31, 0x5b, 0x26, 0x41,
0xa9, 0x15, 0x99, 0xdc, 0x01, 0x0d, 0x15, 0xfd,


*/

