/*
* $Log: think.c,v $
* Revision 1.5  2004/02/14 16:43:06  richq
* the tiredness descriptions for riders and warriors
* were displayed the wrong way around.
*
* Revision 1.4  2004/01/31 15:38:13  Default
* Think screen picture doesn't change, so no need
* to redraw it every time 'think' is pressed.
*
* Revision 1.3  2004/01/27 20:19:13  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.2  2004/01/25 21:30:36  richq
* Think split in 2 to allow objects discovered to be described.
* Battle Stats added.
*
* Revision 1.1  2004/01/22 22:27:09  richq
* He thinks again.... and I thought "better move all this think screen stuff
* to its own .c file". So that's what I did. Was mostly in lords.c before.
*
*
*/

#include "screenmode.h"
#include "think.h"
#include "gfx.h"

#include "location.h"
#include "shield.h"
#include "objects.h"
#include "objectdata.h"
#include "tokens.h"
#include "tokendata.h"
#include "lords.h"
#include "savearea.h"
#include "army.h"
#include "icefear.h"

#include "blit.h"

/* 
clears the screen, the text, removes any objects
sets the palette background to grey/white
*/
void setupInfoScreen(void) {
  // clear top screen
  clear_text();
  // clear bottom
  clear_objects();
  // clear shield...
  remove_shield();
  clear_view();
  CopyOAM();
  set_think_palette();
  set_object_palette(0);
  clearBuffer();
}


void drawThinkPicture(void) {
  // clear screen...
  if (g_lordScreen != THINK_SCREEN) {
    setupInfoScreen();
    drawFeature(g_locdat.terrain, 0, THINK_PIC_X, THINK_PIC_Y);
    wait_vsync_int();
    blit_buffer();
    setup_view();
    
    bg_level.y_scroll = 256-5;
    UpdateBackground(&bg_level);
    
    // now for lords title
    createLordTitleUppercase1(g_selectedLord->title, g_selectedLord->firstname);
    print_tokens(0, 0, 1, 0x17, 2);
  
  
    // and finally the lordly pic..
    drawGFX(g_selectedLord->graphic,7,10-objectGFXTable[g_selectedLord->graphic]->height,OBJECT_SPRITE);
    CopyOAM();
  } else {
    clear_text_from(0xb);
  }
  
  
}

void addEnergyWords(int energy) {
  
  // d6 = invigorated
  // d5 = tired
  addMildlyCold(energy/16, 0xd6, 0xd5);
  
  if (energy==0) {
    addToTokenList(andCannotContinue);
  }
    
}


// f97b
void addTime(int time) {
  if (time < 1) {
    // night..
    addToTokenList(nightTime);
    return;
  }
  int halftime = (time/2);
  if (halftime == 8) {
    // dawn
    addToTokenList(dawnTime);
    return;
  }
  
  // else some time during the day
  if (time & 1) {
    // round up the lost bit
    halftime++;
    addToTokenList(lessThan);
  }
  
  g_isPlural = !(halftime == 1);
  
  addSingleToken(0x62 + halftime); // one, two, etc
  addPluralisedToken(0x62); // hour(s)
  addToTokenList(ofTheDay);
  // pluralise remain(s) so that it is formed correctly for hour or hours:
  // 2 hourS remain_
  // 1 hour_ remainS
  g_isPlural = (halftime == 1);
  addPluralisedToken(0x6d); // remain(s)
  
}


// lord is happy. ice fear is, etc
// ed1b
void thinkStatus(void) {
  setSelectedLord(g_saveArea.selectedLord);
  addUppercaseToken();
  // addtime e.g. "less than two hours of the day remain"
  addTime(g_selectedLord->time);
  // add energy "and Morkin is very tired"
  addSingleToken(0x87);   // and
  addNameToken(g_selectedLord->firstname);
  addSingleToken(0x70);   // is
  addEnergyWords(g_selectedLord->energy);
  
  // add icefear words "The Ice Fear is slightly mild"
  addToTokenList(theIcefearIs);
  addMildlyCold(7 - icefear/64, 0xf5, 0xf4); 
  addLiteralToken('.');
  
  // add courage words "Luxor is utterly bold"
  addNameToken(g_selectedLord->firstname);
  addSingleToken(0x70);   // is
  addMildlyCold(g_selectedLord->courage, 0xf3, 0xf2); 
  addLiteralToken('.');
  
  if (g_selectedLord->object) {
    // "He has with him the sword Wolfslayer"
    addToTokenList(heHasWithHim);
    addToTokenList(objectDescriptions[g_selectedLord->object]);
    addLiteralToken('.');
  }
  
}

// code from ec8d
void doFirstPage(void) {
  
  // check hidden
  if (g_selectedLord->hidden) {
    addNameToken(g_selectedLord->firstname);
    addToTokenList(ishidden);
  }
  addUppercaseToken();
  // check nasties, found object etc
  thinkAboutObjects();
  // describe condition
  // this is ed1b...
  thinkStatus();
  
  
  
}

void addSoldierKills(int killed) {
  if (killed == 0) {
    addSingleToken(0xcc);
  }
  else {
    numberToWords(killed*5);
  }
  
}


void showBattleStats(void) {
  // dfa8
  addToTokenList(inTheBattleOf);
  addAreaToken(g_selectedLord->battle_area);
  addLiteralToken(',');
  int flag = 0;
  if (!g_selectedLord->riders && !g_selectedLord->riders_lost) {
    // no need to talk about riders
    flag|=1;
  }
  if (!g_selectedLord->warriors && !g_selectedLord->warriors_lost) {
    // no need to talk about warriors
    flag|=2;
  }
  if (flag != 3) {
    // lost stuff
    addNameToken(g_selectedLord->firstname);
    addSingleToken(0x07);
    if ((flag&1) == 0) {
      // talk about riders
      numberToWords(g_selectedLord->riders_lost*5);
      addSingleToken(0xc4); //rider
      addLiteralToken('s');
    }
    if (flag == 0) {
      addSingleToken(0x87); //and
    }
    if ((flag&2) == 0) {
      // talk about warriors
      numberToWords(g_selectedLord->warriors_lost*5);
      addSingleToken(0xc5); //warrior
      addLiteralToken('s');
    }
    addLiteralToken('.');
  }
  addNameToken(g_selectedLord->firstname);
  addToTokenList(aloneSlew);
  addSoldierKills(g_selectedLord->killed);
  addToTokenList(ofTheEnemy);
  
  if (flag != 3) {
    // need to discuss riders/warriors
    if (flag != 1) {
      // riders
      addToTokenList(hisRidersSlew);
      addSoldierKills(g_selectedLord->riders_killed);
      addLiteralToken('.');
    }
    
    if (flag != 2) {
      // riders
      addToTokenList(hisWarriorsSlew);
      addSoldierKills(g_selectedLord->warriors_killed);
      addLiteralToken('.');
    }
  }
  
  if (!g_selectedLord->battle_status) {
    addToTokenList(theBattleContinues);
  } else {
    addToTokenList(victoryWent);
    addSingleToken(g_selectedLord->battle_status);
    addLiteralToken('!');
  }
}


int g_lordToShow;
int g_thinkSubpage;
int g_thinkCheckInfront;

void resetThinkingFlags(void) {
    g_lordToShow = 0;
    g_thinkSubpage = 0;
    g_thinkCheckInfront = 0;
}

void addLordCommands(void) {
  
  createLordTitle(g_selectedLord->title, g_selectedLord->firstname); 
  addSingleToken(0xd7);
  addLiteralToken('s');
  int flags = 0;
  int joiner = 0x87; // and
  if (g_selectedLord->riders)
    flags |= 1;
  if (g_selectedLord->warriors)
    flags |= 2;
  
  if (flags != 0 && flags != 3)
    joiner = 0xce; // but
  
  numberToWords(g_selectedLord->warriors*5);
  addSingleToken(0xc5); //warrior
  addLiteralToken('s');
  addSingleToken(joiner); // and, but
  numberToWords(g_selectedLord->riders*5);
  addSingleToken(0xc4); //rider
  addLiteralToken('s'); 
  addLiteralToken('.'); 
  
  if (flags == 0) //neither warriors nor riders
    return;
  
  if (flags != 1) {
    addToTokenList(hisWarriorsAre);
    addEnergyWords(g_selectedLord->warriors_energy);
    addLiteralToken('.'); 
  }
  if (flags == 2)
    return;
  addToTokenList(hisRidersAre);
  addEnergyWords(g_selectedLord->riders_energy);
  addLiteralToken('.'); 

}


// returns whether we need to restart or not
int pageThree(void) {
  //eec2
  if (g_thinkCheckInfront != 1) {
    // need to look at the square in front of us
    workOutLocationDetails(&g_inFront);
  }
  if (g_lordToShow >= g_lordsHere) {
    if (g_thinkCheckInfront == 1) {
      // we have gone through everyone where we are
      resetThinkingFlags();
    } else {
      // erm..
      g_thinkCheckInfront--;
      g_lordToShow = 0;
    }
    return 1;
  }
  else {
    setSelectedLord(g_charTable[g_lordToShow]);
    
    if (g_thinkCheckInfront == 1) {
      addUppercaseToken();
    } else {
      addToTokenList(toThe);
      addDirection(g_inFront.direction);
      addLiteralToken(',');
      
    }
    // add info on lord
    // e103..
    addLordCommands();
    g_lordToShow++;
    return 0;
  }
}


void doThinkText(void) {
  
  if (g_lordScreen != THINK_SCREEN) {
    g_lordScreen = THINK_SCREEN;
    resetThinkingFlags();
  }
  updateCurrentInfo();
  if (!g_selectedLord->alive) {
    // describe death...
    addUppercaseToken();
    if (g_selectedLord->deathby != 0) {
      addToTokenList(objectDescriptions[g_selectedLord->deathby]);
      addToTokenList(slewhim);
    } else {
      // was killed in battle
      addToTokenList(killedInBattle);
      showBattleStats();
    }
    return;
  }
  // else alive
  if (g_thinkSubpage == 0) {
    g_thinkSubpage++;
    doFirstPage();
  } else if (g_thinkSubpage == 1) {
    g_thinkSubpage++;
    if (g_selectedLord->battle_status == 0xff) {
      // nope, next!
      doThinkText();
      return;
    } else {
      showBattleStats();
    }
  } else if (g_thinkSubpage == 2) {
    g_thinkCheckInfront++;
    if (g_thinkCheckInfront != 1) {
      // will go here after the second time through...
      // check occupied location infront
      // call eef4
      workOutLocationDetails(&g_inFront);
      g_thinkSubpage = 3;
    }
    // else == 1
    if (g_locdat.terrain != CITADEL &&
        g_locdat.terrain != KEEP &&
        g_doomindex == 0) {
      // nope... there is no way an army is in front of us
      doThinkText();
      return;
    
    }
    // keep infront or doomdarks there
    // eebc
    if (g_thinkCheckInfront == 1) {
      addUppercaseToken();
    } else {
      addToTokenList(toThe);
      addDirection(g_inFront.direction);
      addLiteralToken(',');
      
    }
    // "five hundred warriors of the utarg guard the citadel"
    addArmyTokens();
  } else {
    // page three returns true if we need to start over
    if (pageThree())
      doThinkText();
  }
  
  
}

/*
He /thinks/ again... 
"\nHe" is the first part - all the previous tokens are drawn to the screen too.
*/
void heThinksAgain(void) {
  addToTokenList(newlineHe);
  // print location taken from fcfd: 
  // 00 0b 00 1f 3a 00 ; starty, startx, indentx, maxx, pen, paper
  print_tokens(0xb, 0, 0, 0x1e, 2);
  addSingleToken(0xc9); // thinks
  print_tokens_pal(4);
  addToTokenList(againDotDotDot);
  print_tokens_pal(2);
}


void thinkWithObjectSet(void) {
  // fc6a update display
  drawThinkPicture();

  // now a lot of text is added to the buffer
  doThinkText();
  // this text is finally decompressed here:
  heThinksAgain();
}

//637b
void think(void) {
  updateCurrentInfo();
  calculateCourage();
  setObjectHere();
  
  thinkWithObjectSet();
}

