/*
*  $Log: title.c,v $
*  Revision 1.1  2004/01/27 19:38:31  richq
*  Title screen - the data for this is the Speccy loading screen.
*
*/

#include "gba.h"
#include "keypad.h"
#include "gfx.h"
#include "screenmode.h"

// comes from the .tap file, offset 245 (0xf5).
#define titleData _binary_title_bin_start
extern const u8 titleData[];


#define SPLASH_WIDTH_FUDGE 256
#define SPLASH_HEIGHT_FUDGE 192

const unsigned short inkMasks[4][2] = {
  { 0x0001,0xFFF0},
  { 0x0010,0xFF0F},
  { 0x0100,0xF0FF},
  { 0x1000,0x0FFF} 
};

// modified from chaos..
// assumes that the tiles are set up with tile 1 being top left
// and in rows of 256
void drawPixel(int  x, int y, int colourIndex) {
  u16 tile = 1 + (x>>3) + ((y>>3)*32);
  u8 xrem = x&7;
  u8 yrem = y&7;
  
  u16 address = tile*0x10 /*16 16-bit vals per tile*/ + (xrem>>2) + (yrem*2);
  
  u16 pixel =  bg_level.tileData[address];
  
  pixel &= (inkMasks[x&3])[1];
  pixel |= (inkMasks[x&3][0])*colourIndex;  
  
  bg_level.tileData[address] = pixel;
  
}


void setTilesForTitle(void) {
  
  SetMode(MODE_0 | BG0_ENABLE);
  
  bg_level.number = 0;				//background number 0-2
  bg_level.charBaseBlock = 0;
  bg_level.screenBaseBlock = 31;
  bg_level.colorMode = BG_COLOR_16;
  bg_level.size = TEXTBG_SIZE_256x256; 
  bg_level.mosiac = 0;
  bg_level.x_scroll = 8;
  bg_level.y_scroll = 16;
  EnableBackground(&bg_level);
  REG_BG0CNT &= 0xFFFD;
  REG_BG0CNT |= 1;
  UpdateBackground(&bg_level);
  
  int x,y;
  int index = 1;
  for (y = 0; y < 24; y++) 
    for (x = 0; x < 32; x++) 
      bg_level.mapData[x+y*32] = index++;
  
}

void createTitleImage(const u8 * const imageData) {
  
  // looks a lot like tile stuff
  const u8 * tmpData = imageData;
  const u8 * paletteBase;
  
  // draw straight to vram
  // int pixels [] = new int[size.width*size.height];
  int y, x, n;
  int black = 1;
  int blue = 0;
  
  int yIndex, pixel;
  int paletteEntryHere;
  for (y = 0; y < SPLASH_HEIGHT_FUDGE; y++) {
    yIndex = y;
    // not exactly y, in fact it is something hideous like:
    // y & 7 + (y/8)&0x18
    
    // gets the row..
    yIndex = ((y & 7) + ((y>>3)&0x18))*8;
    yIndex +=((y/8) & 7);
    //palette base index for this row
    // assumes pic is 256*192 / 8 bytes big
    // and that palette stuff immediately follows
    paletteBase = imageData + 0x1800 + (yIndex&0xf8)*4;
  
    for (x = 0; x < SPLASH_WIDTH_FUDGE/8; x++) {
      // read in the byte of gfx data...
      u8 data = *tmpData++;//byteAt(index++);
      // read in palette
      paletteEntryHere = *(paletteBase + x);
      blue = paletteEntryHere&7;
      black = (paletteEntryHere>>3)&0x7;
      
      // now expand to 8 pixels
      for (n = 0; n < 8; n++) {
        //int pixelIndex = x*8 + yIndex*width*8 + n;
        
        if (n != 7)
          pixel = data & (1<<(7-n));
        else
          pixel = data & 1;
        
        if (pixel == 0) 
          drawPixel(n+x*8,yIndex, black+1);
        else
          drawPixel(n+x*8,yIndex, blue+1);
      }
    }
  }
  
}

const u16 titlePal[] = {
  RGB16(0,0,0),         //black,
  RGB16(0,0,30),        //blue,
  RGB16(30,0,0),        //red,
  RGB16(30,0,30),       // purple
  RGB16(0,30,0),        //green,
  RGB16(0,30,30),       // cyan 
  RGB16(30,30,0),       // yellow
  RGB16(30,30,30),      // white
};

// shields are sprites.
void init_title_pal(void) {
  int i;
  for (i = 0 ; i < 8; i++) {
    BGPaletteMem[i+1] = titlePal[i];
  }
  
}

void showSplash(void) {
  // first initialise tile ram...
  fade_up();
  setTilesForTitle();
  init_title_pal();
  
  // create the image
  createTitleImage(titleData);
  
  // loop on keys...
  WaitKeyPress(KEY_START);
  WaitKeyReleased(KEY_START);
  fade_down();
}

