/*
* $Log: tokendata.c,v $
* Revision 1.5  2004/02/18 20:02:14  richq
* const fix for object descriptions
*
* Revision 1.4  2004/01/25 21:31:07  richq
* Tokens for night time and battles
*
* Revision 1.3  2004/01/22 20:29:48  richq
* Yet more token arrays..
*
* Revision 1.2  2004/01/22 19:55:15  richq
* More token arrays added!
*                                                        
* Revision 1.1  2004/01/21 19:30:31  Default                                   
* Arrays of tokens that are used in various places.                            
* Generally they are only used once, so it doesn't really matter where they go,
* but having them all in one place is tidier.                                  
*                                                                              
*/



#include "tokendata.h"

// at, on, in etc
const u8 phrasalEndings[] = { 0x54, 0x5d, 0x54, 0x5d, 0x5d, 0x54, 0x5e, 0x5d, 
                              0x5d, 0x5f, 0x54, 0x54, 0x5d, 0x5d, 0x54, 0x5e};

const u8 heStands[] = {0xfc, 0x55, 0x56, 0xfe, 0x73, 0xff};

const u8 directions[][2] = {
  {0x57, 0x00},
  {0x57, 0x59},
  {0x59, 0x00},
  {0x58, 0x59},
  {0x58, 0x00},
  {0x58, 0x5a},
  {0x5a, 0x00},
  {0x57, 0x5a}
};


  
const u8 theTeens[] = {  
  0x6c,      // ; ten   
  0xb3,      // ; eleven
  0xb4,      // ; twelve
  0xb5,      // ; thir  
  0x66,      // ; four  
  0xb7,      // ; fif   
  0x68,      // ; six   
  0x69,      // ; seven 
  0x6a,      // ; eight 
  0x6b       // ; nine  
};

const u8 theTees[] = {  
  0xb9,      // ; twenty
  0xb5,      // ; thir  
  0xb6,      // ; for   
  0xb7,      // ; fif   
  0x68,      // ; six   
  0x69,      // ; seven 
  0x6a,      // ; eight 
  0x6b       // ; nine  
};

const u8 een[] = {  
  0xfe, 0x65, 0xfe, 0x65, 0xfe, 0x6e, 0xff
};
const u8 dashJoin[] = {  
  0xfe, 0x2d, 0xfd, 0xff
};

const u8 lake[] = {0xfc, 0x4a, 0xfc, 0xff};
const u8 wastes[] = {0x00,0xfc,0x51,0xfc,0x4b,0xff};
const u8 indomof [] = {0x54,0x00,0xfc,0x52,0x40,0xff};

const u8 theLordOf[] = {0x00, 0xfc, 0x79, 0x40, 0xff}; // the Lord of  
const u8 Lord[] = {0xfc, 0x79, 0xff}; //Lord

const u8 ishidden[] = {0x70, 0xef, 0xfe, 0x2e, 0xff}; 

const u8 newlineHe[] = {0xfb, 0xfc, 0x55, 0xff}; // newline, He
const u8 againDotDotDot[]= {0x94, 0xfe, 0x2e, 0xfe, 0x2e, 0xfe, 0x2e, 0xfe, 0x2e, 0xff}; // again....


const u8 areAbroad[] = { 0xae, 0xad, 0xff}; // are abroad
const u8 areAbroadNasty[] = { 0xae, 0xad, 0x87, 0x55,
                              0xa4, 0xf7, 0xa8, 0xee, 
                              0xff}; // are abroad and he must fight or hide
                              
/*The following is the object description table from ee2a*/
const u8 objdesc01[] =  {0xb0, 0xff};                           // nothing
const u8 objdesc02[] =  {0xf8, 0xff};                           // wolves                                            
const u8 objdesc03[] =  {0x7d, 0xfe, 0x73, 0xff};               // dragons                                           
const u8 objdesc04[] =  {0x9b, 0x9c, 0xfe, 0x73, 0xff};         // ice trolls                                        
const u8 objdesc05[] =  {0x7c, 0xff};                           // skulkrin
const u8 objdesc06[] =  {0x9d, 0x99, 0xfe, 0x73, 0xff};         // wild horses                                       
const u8 objdesc07[] =  {0xdc, 0x87, 0x70, 0xdd, 0xff};         // shelter and is refreshed
const u8 objdesc08[] =  {0xd9, 0xfe, 0x2e, 0xfc, 0x53, 0xda, 0xdb, 0xfe, 0x2c, 0xff};// guidance. A voice calls,                          
const u8 objdesc09[] =  {0x00, 0xfc, 0x1a, 0x40, 0xfc, 0x0b, 0xde, 0xdf, 0xf9, 0x40, 0xe1, 0xff};                    // the Shadows of Death which drain him of vigour    
const u8 objdesc10[] =  {0x00, 0xfc, 0xe8, 0x40, 0xfc, 0xe2, 0xde, 0xe0, 0xf9, 0xe3, 0xe1, 0xff};                    // the Waters of Life which fill him with vigour     
const u8 objdesc11[] =  {0x00, 0xfc, 0xe4, 0x40, 0x00, 0xfc, 0xa0, 0xde, 0xe5, 0x0b, 0x5c, 0x00, 0x6e, 0xff};                    // the Hand of the Dark which brings death to the day
const u8 objdesc12[] =  {0x00, 0xfc, 0xe9, 0x40, 0xfc, 0x32, 0xde, 0xe5, 0xa7, 0xa2, 0xff};                               // the Cup of Dreams which brings new welcome        
const u8 objdesc13[] =  {0x00, 0xe6, 0xfc, 0x9a, 0xfd, 0xe7, 0xff};       // the sword Wolfslayer                              
const u8 objdesc14[] =  {0x00, 0xe6, 0xfc, 0x7d, 0xfd, 0xe7, 0xff};       // the sword Dragonslayer                            
const u8 objdesc15[] =  {0x00, 0xfc, 0x9b, 0xfc, 0xea, 0xff};             // the Ice Crown                                     
const u8 objdesc16[] =  {0x00, 0xfc, 0x03, 0xfc, 0xfa, 0xff};             // the Moon Ring                                     
const u8 objdesc17[] =  {0xfc, 0x76, 0x00, 0xfc, 0x7c, 0xff};             // Fawkrin the Skulkrin                              
const u8 objdesc18[] =  {0xfc, 0x77, 0x00, 0xfc, 0x7d, 0xfd, 0x79, 0xff}; // Farflame the Dragonlord                           
const u8 objdesc19[] =  {0xfc, 0x4a, 0xfc, 0x04, 0xff};                   // Lake Mirrow                                       
const u8 objdesc20[] =  {0xfc, 0x2d, 0x00, 0xfc, 0x7b, 0xff};             // Lorgrim the Wise                                  

const u8 * const objectDescriptions[20] = {
  objdesc01,
  objdesc02,
  objdesc03,
  objdesc04,
  objdesc05,
  objdesc06,
  objdesc07,
  objdesc08,
  objdesc09,
  objdesc10,
  objdesc11,
  objdesc12,
  objdesc13,
  objdesc14,
  objdesc15,
  objdesc16,
  objdesc17,
  objdesc18,
  objdesc19,
  objdesc20
};


const u8 hehasfound[] = {0xfc, 0x55, 0x8a, 0xaf, 0xff}; // He has found
const u8 heslewthe[] = {0x55, 0xc7, 0x00, 0xff}; //he slew the

const u8 canDestroyCrown[] = {0xed, 0xeb, 0x00, 0xfc, 0x9b, 0xfc, 0xea, 0xff}; // "can destroy the Ice Crown"

const u8 lookingFor[] = {0xfc, 0x5b, 0xb6, 0xff};

const u8 youMustSeek[] = {0x8d, 0xa4, 0xf0, 0xff}; 

const u8 nightTime[] = {0x6f, 0x70, 0x71, 0xff}; // it is night
const u8 dawnTime[] = {0x6f, 0x70, 0x3d, 0xff}; // it is dawn
const u8 lessThan[] = {0x60, 0x61, 0xff}; // less than
const u8 ofTheDay[] = {0x40, 0x00, 0x6e, 0xff};

const u8 andCannotContinue[] = {0x87, 0xd8, 0x92, 0xff};
const u8 theIcefearIs[] = {0xfe, 0x2e, 0xfc, 0x00, 0xfc, 0x9b, 0xfc, 0xa9, 0x70, 0xff}; // ". The Ice Fear is"
const u8 heHasWithHim[] = {0xfc, 0x55, 0x8a, 0xe3, 0xf9, 0xff};

const u8 toThe[] = {0xfc, 0x5c, 0x00, 0xff};

const u8 doomdarkHasArmy[] = {0xfc, 0x0c, 0xfd, 0xa0, 0x8a, 0x95, 0x4f, 0x40, 0xff};

const u8 ofthe[] = {0x40, 0x00, 0xfc, 0xff};

const u8 doomGrdTokens[] = {0x0c, 0xfd, 0x9e, 0xff };
const u8 guardThe[] = {0x9e, 0x00, 0xff};
 
const u8 killedInBattle[] = {0x55, 0x90, 0xf1, 0x54, 0xc3, 0xfe, 0x2e, 0xff};
const u8 slewhim[] = {0xc7, 0xf9, 0xfe, 0x2e, 0xff };

const u8 hisWarriorsAre[] = { 0xfc, 0xcf, 0xc5, 0xfe, 0x73, 0xae, 0xff };
const u8 hisRidersAre[] = {0xfc, 0xcf, 0xc4, 0xfe, 0x73, 0xae, 0xff };

// might need to alter this one...
const u8 press2choose[] = {0xfc, 0x7f, 0x53, 0x81, 0x5c, 0x80, 0xfe, 0x2e, 
                           0xfe, 0x2e, 0xfe, 0x2e, 0xfe, 0x2e, 0xfb, 0xfb, 
                           0xff}; // Press a key to choose....

const u8 donothide[] = {0x8c, 0xbc, 0xfe, 0x74, 0xee, 0xff};
const u8 fightThe[] = {0xf7, 0x00, 0xff}; 
const u8 recruitmen[] = {0xf6, 0xcd, 0xff}; 
const u8 standOnGuard[] = {0x56, 0xcd, 0x5e, 0x9e, 0xff};

const u8 tobattle[] = {0x5c, 0xc3, 0xfe, 0x21, 0xff};

const u8 nightHasFallen[] = {
     0xfc, 0x71, 0x8a, 0xab, 0x87, 0x00, 0xfc, 0xac,
     0xae, 0xad, 0xfe, 0x21, 0xfc, 0xff };
     
const u8 passedSinceWarBegan[] = {
     0xbe, 0xfe, 0x65, 0xfe, 0x64, 0xc2, 0x00, 0xfc, 0xbf, 
     0x40, 0x00, 0xfc, 0xc0, 0xc1, 0xfe, 0x2e, 0xff};
     
const u8 dyawantDawn[] = {0xfb, 0xfc, 0x8c, 0x8d, 0x8e, 0x3d, 0xfe, 0x3f, 0xff }; //Do you want dawn?

const u8 luxorsdead[] = {0xfc, 0x72, 0x70, 0x08, 0xff};
const u8 andMorkinIsDead[] = {
     0x87, 0xfc, 0x73, 0x70, 0x08, 0xfe, 0x2e, 0xfc, 
     0xca, 0x5c, 0xfc, 0x0c, 0xfd, 0xa0, 0xfe, 0x21, 
     0xff };
     
const u8 xajHasFallen[] = {0xfc, 0x35, 0x8a, 0xab, 0xff };

const u8 ushHasFallen[] = {0xfc, 0x0f, 0x8a, 0xab, 0xff };

const u8 victoryToFree[] = {
     0xfe, 0x2e, 0xfc, 0xca, 0x5c, 0x00, 0xfc, 0x9f, 
     0xfe, 0x21, 0xff };
const u8 icecrownDestroyed[] = {     
     0xfc, 0x00, 0xfc, 0x9b, 0xfc, 0xea, 0x8a, 0x8b, 
     0xeb, 0xfe, 0x65, 0xfe, 0x64, 0xff };
     
const u8 theBloodySword[] = {
     0xfb, 0xfc, 0x00, 0x1b, 0xfe, 0x79, 0xe6, 0x40, 
     0xc3, 0xe5, 0x0b, 0x54, 0x00, 0xfc, 0x52, 0xff };
     
const u8 inTheBattleOf[] = {
     0xfc, 0x54, 0x00, 0xfc, 0xc3, 0x40, 0xff}; //In the Battle of
     

const u8 aloneSlew[] = {0xc6, 0xc7, 0xff};
const u8 ofTheEnemy[] = {0x40, 0x00, 0xfc, 0xc8, 0xfe, 0x2e, 0xff}; //of the Enemy.
const u8 hisRidersSlew[] = {0xfc, 0xcf, 0xc4, 0xfe, 0x73, 0xc7, 0xff }; //His riders slew
const u8 hisWarriorsSlew[] = {0xfc, 0xcf, 0xc5, 0xfe, 0x73, 0xc7, 0xff }; //His warriors slew
const u8 theBattleContinues[] = {0xfc, 0x00, 0xc3, 0x92, 0xfe, 0x73, 0xfe, 0x21, 0xff}; //The battle continues!
const u8 victoryWent[] = {0xfc, 0xca, 0xcb, 0x5c, 0x00, 0xfc, 0xff};

